# Copyright 2017 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import mock
from six import BytesIO

from glare.common import exception as exc
from glare.common import store_api
from glare.db import artifact_api
from glare.tests.unit import base


class TestArtifactDelete(base.BaseTestArtifactAPI):

    """Test Glare artifact deletion."""

    def setUp(self):
        super(TestArtifactDelete, self).setUp()
        values = {'name': 'ttt', 'version': '1.0'}
        self.artifact = self.controller.create(
            self.req, 'sample_artifact', values)
        # Upload data
        self.controller.upload_blob(
            self.req, 'sample_artifact', self.artifact['id'], 'blob',
            BytesIO(b'a' * 100), 'application/octet-stream')
        # Check that data was uploaded successfully
        self.artifact = self.controller.show(
            self.req, 'sample_artifact', self.artifact['id'])
        self.assertEqual(100, self.artifact['blob']['size'])
        self.assertEqual('active', self.artifact['blob']['status'])

    @mock.patch('glare.common.store_api.delete_blob',
                side_effect=store_api.delete_blob)
    def test_delete_with_data(self, mocked_delete):
        # Delete artifact and check that 'delete_blob' was called
        self.controller.delete(self.req, 'sample_artifact',
                               self.artifact['id'])
        self.assertRaises(exc.NotFound, self.controller.show,
                          self.req, 'sample_artifact', self.artifact['id'])
        self.assertEqual(1, mocked_delete.call_count)

    @mock.patch('glare.common.store_api.delete_blob',
                side_effect=store_api.delete_blob)
    def test_delete_with_blob_dict(self, mocked_delete):
        # Upload data
        for i in range(10):
            self.controller.upload_blob(
                self.req, 'sample_artifact', self.artifact['id'],
                'dict_of_blobs/blob%d' % i,
                BytesIO(b'a' * 100), 'application/octet-stream')
        # Check that data was uploaded successfully
        self.artifact = self.controller.show(
            self.req, 'sample_artifact', self.artifact['id'])
        for i in range(10):
            self.assertEqual(
                100,
                self.artifact['dict_of_blobs']['blob%d' % i]['size'])
            self.assertEqual(
                'active',
                self.artifact['dict_of_blobs']['blob%d' % i]['status'])
        # Delete artifact and check that 'delete_blob' was called for each blob
        # 10 times for blob dict elements and once for 'blob'
        self.controller.delete(self.req, 'sample_artifact',
                               self.artifact['id'])
        self.assertRaises(exc.NotFound, self.controller.show,
                          self.req, 'sample_artifact', self.artifact['id'])
        self.assertEqual(11, mocked_delete.call_count)

    def test_delete_not_found(self):
        self.assertRaises(exc.NotFound, self.controller.delete,
                          self.req, 'sample_artifact', 'INVALID_ID')

    def test_delete_saving_blob(self):
        blob = self.artifact['blob']
        # Change status of the blob to 'saving'
        blob['status'] = 'saving'
        artifact_api.ArtifactAPI().update_blob(
            self.req.context, self.artifact['id'], {'blob': blob})
        self.artifact = self.controller.show(
            self.req, 'sample_artifact', self.artifact['id'])
        blob = self.artifact['blob']
        self.assertEqual(100, blob['size'])
        self.assertEqual('saving', blob['status'])
        # Deleting of the artifact leads to Conflict error
        self.assertRaises(exc.Conflict, self.controller.delete,
                          self.req, 'sample_artifact', self.artifact['id'])
        self.artifact = self.controller.show(
            self.req, 'sample_artifact', self.artifact['id'])
        self.assertEqual('drafted', self.artifact['status'])

    def test_delete_deleted_artifact(self):
        # Change status of the artifact to 'deleted'
        artifact_api.ArtifactAPI().save(
            self.req.context, self.artifact['id'], {'status': 'deleted'})
        # Delete should work properly
        self.controller.delete(self.req, 'sample_artifact',
                               self.artifact['id'])
        self.assertRaises(exc.NotFound, self.controller.show,
                          self.req, 'sample_artifact', self.artifact['id'])

    @mock.patch('glare.common.store_api.delete_blob',
                side_effect=exc.NotFound)
    def test_delete_link_not_exist(self, mocked_delete):
        # Delete artifact and check that 'delete_blob' was called
        self.controller.delete(self.req, 'sample_artifact',
                               self.artifact['id'])
        self.assertRaises(exc.NotFound, self.controller.show,
                          self.req, 'sample_artifact', self.artifact['id'])
        self.assertEqual(1, mocked_delete.call_count)

    @mock.patch('glare.common.store_api.delete_blob',
                side_effect=exc.Forbidden)
    def test_no_delete_permission(self, mocked_delete):
        # Try to delete artifact
        self.assertRaises(exc.Forbidden, self.controller.delete,
                          self.req, 'sample_artifact', self.artifact['id'])

    @mock.patch('glare.common.store_api.delete_blob',
                side_effect=exc.GlareException)
    def test_delete_unknown_store_exception(self, mocked_delete):
        # Try to delete artifact
        self.assertRaises(exc.GlareException, self.controller.delete,
                          self.req, 'sample_artifact', self.artifact['id'])

    @mock.patch('glare.common.store_api.delete_blob',
                side_effect=exc.NotFound)
    def test_delete_blob_not_found(self, mocked_delete):
        # Upload a file to blob dict
        self.controller.upload_blob(
            self.req, 'sample_artifact', self.artifact['id'],
            'dict_of_blobs/blob',
            BytesIO(b'a' * 100), 'application/octet-stream')

        # Despite the exception artifact should be deleted successfully
        self.controller.delete(self.req, 'sample_artifact',
                               self.artifact['id'])
        self.assertRaises(exc.NotFound, self.controller.show,
                          self.req, 'sample_artifact', self.artifact['id'])
        self.assertEqual(2, mocked_delete.call_count)

    @mock.patch('glare.common.store_api.delete_blob',
                side_effect=store_api.delete_blob)
    def test_delayed_delete_global(self, mocked_delete):
        # Enable delayed delete
        self.config(delayed_delete=True)
        # Delete artifact and check that 'delete_blob' was not called
        self.controller.delete(self.req, 'sample_artifact',
                               self.artifact['id'])
        self.assertEqual(0, mocked_delete.call_count)
        # Check that artifact status is 'deleted' and its blob is
        # 'pending_delete'
        self.artifact = self.controller.show(
            self.req, 'sample_artifact', self.artifact['id'])
        self.assertEqual('deleted', self.artifact['status'])
        self.assertEqual('active', self.artifact['blob']['status'])
        # Disable delayed delete
        self.config(delayed_delete=False)
        # Delete artifact and check that 'delete_blob' was called this time
        self.controller.delete(self.req, 'sample_artifact',
                               self.artifact['id'])
        self.assertEqual(1, mocked_delete.call_count)
        self.assertRaises(exc.NotFound, self.controller.show,
                          self.req, 'sample_artifact', self.artifact['id'])

    @mock.patch('glare.common.store_api.delete_blob',
                side_effect=store_api.delete_blob)
    def test_delayed_delete_per_artifact_type(self, mocked_delete):
        # Enable delayed delete for sample_artifact type
        # Global parameter is disabled
        self.config(delayed_delete=True,
                    group='artifact_type:sample_artifact')
        # Delete artifact and check that 'delete_blob' was not called
        self.controller.delete(self.req, 'sample_artifact',
                               self.artifact['id'])
        self.assertEqual(0, mocked_delete.call_count)
        # Check that artifact status is 'deleted' and its blob is
        # 'pending_delete'
        self.artifact = self.controller.show(
            self.req, 'sample_artifact', self.artifact['id'])
        self.assertEqual('deleted', self.artifact['status'])
        self.assertEqual('active', self.artifact['blob']['status'])
        # Disable delayed delete
        self.config(delayed_delete=False,
                    group='artifact_type:sample_artifact')
        # Delete artifact and check that 'delete_blob' was called this time
        self.controller.delete(self.req, 'sample_artifact',
                               self.artifact['id'])
        self.assertEqual(1, mocked_delete.call_count)
        self.assertRaises(exc.NotFound, self.controller.show,
                          self.req, 'sample_artifact', self.artifact['id'])



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import sqlalchemy
import testtools

from glare.common import exception
from glare.tests.unit import glare_fixtures


class TestBannedDBSchemaOperations(testtools.TestCase):
    def test_column(self):
        column = sqlalchemy.Column()
        with glare_fixtures.BannedDBSchemaOperations(['Column']):
            self.assertRaises(exception.DBNotAllowed,
                              column.drop)
            self.assertRaises(exception.DBNotAllowed,
                              column.alter)

    def test_table(self):
        table = sqlalchemy.Table()
        with glare_fixtures.BannedDBSchemaOperations(['Table']):
            self.assertRaises(exception.DBNotAllowed,
                              table.drop)
            self.assertRaises(exception.DBNotAllowed,
                              table.alter)



# Copyright 2017 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from glare.db.sqlalchemy import api as db_api
from glare.store import base_api


class DatabaseStoreAPI(base_api.BaseStoreAPI):
    """Class that stores all data in sql database."""

    def add_to_backend(self, blob_id, data, context, verifier=None):
        session = db_api.get_session()
        return db_api.save_blob_data(context, blob_id, data, session)

    def add_to_backend_batch(self, blobs, context, verifier=None):
        session = db_api.get_session()
        return db_api.save_blob_data_batch(context, blobs, session)

    def get_from_store(self, uri, context):
        session = db_api.get_session()
        return db_api.get_blob_data(context, uri, session)

    def delete_from_store(self, uri, context):
        session = db_api.get_session()
        return db_api.delete_blob_data(context, uri, session)



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_versionedobjects import fields

from glare.common import exception
from glare.objects import base
from glare.objects.meta import wrappers


Field = wrappers.Field.init


class All(base.BaseArtifact):
    """Artifact type that allows to get artifacts regardless of their type"""

    fields = {
        'type_name': Field(fields.StringField,
                           description="Name of artifact type."),
    }

    @classmethod
    def create(cls, context):
        raise exception.Forbidden("This type is read only.")

    def save(self, context):
        raise exception.Forbidden("This type is read only.")

    @classmethod
    def delete(cls, context, af):
        raise exception.Forbidden("This type is read only.")

    @classmethod
    def update_blob(cls, context, af_id, field_name, values):
        raise exception.Forbidden("This type is read only.")

    @classmethod
    def get_type_name(cls):
        return "all"



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Database API for all artifact types"""

from oslo_db import exception as db_exception
from oslo_log import log as logging
from retrying import retry

from glare.db.sqlalchemy import api
from glare import locking

LOG = logging.getLogger(__name__)


def _retry_on_connection_error(exc):
    """Function to retry a DB API call if connection error was received."""

    if isinstance(exc, db_exception.DBConnectionError):
        LOG.warning("Connection error detected. Retrying...")
        return True
    return False


class ArtifactAPI(object):

    def _serialize_values(self, values):
        new_values = {}
        if 'tags' in values:
            new_values['tags'] = values.pop('tags') if values['tags'] else []
        for key, value in values.items():
            if key in api.BASE_ARTIFACT_PROPERTIES:
                new_values[key] = value
            else:
                new_values.setdefault('properties', {})[key] = value
        return new_values

    @retry(retry_on_exception=_retry_on_connection_error, wait_fixed=1000,
           stop_max_attempt_number=20)
    def save(self, context, artifact_id, values):
        """Save artifact values in database

        :param artifact_id: id of artifact that needs to be updated
        :param context: user context
        :param values: values that needs to be updated
        :return: dict of updated artifact values
        """
        session = api.get_session()
        return api.create_or_update(
            context, artifact_id, self._serialize_values(values), session)

    @retry(retry_on_exception=_retry_on_connection_error, wait_fixed=1000,
           stop_max_attempt_number=20)
    def update_blob(self, context, artifact_id, values):
        """Create and update blob records in db

        :param artifact_id: id of artifact that needs to be updated
        :param context: user context
        :param values: blob values that needs to be updated
        :return: dict of updated artifact values
        """
        session = api.get_session()
        return api.create_or_update(
            context, artifact_id, {'blobs': values}, session)

    @retry(retry_on_exception=_retry_on_connection_error, wait_fixed=1000,
           stop_max_attempt_number=20)
    def delete(self, context, artifact_id):
        """Delete artifacts from db

        :param context: user context
        :param artifact_id: id of artifact that needs to be deleted
        """
        session = api.get_session()
        api.delete(context, artifact_id, session)

    @retry(retry_on_exception=_retry_on_connection_error, wait_fixed=1000,
           stop_max_attempt_number=20)
    def get(self, context, artifact_id):
        """Return artifact values from database

        :param context: user context
        :param artifact_id: id of the artifact
        :return: dict of artifact values
        """
        session = api.get_session()
        return api.get(context, artifact_id, session)

    @retry(retry_on_exception=_retry_on_connection_error, wait_fixed=1000,
           stop_max_attempt_number=20)
    def list(self, context, filters, marker, limit, sort, latest):
        """List artifacts from db

        :param context: user request context
        :param filters: filter conditions from url
        :param marker: id of first artifact where we need to start
        artifact lookup
        :param limit: max number of items in list
        :param sort: sort conditions
        :param latest: flag that indicates, that only artifacts with highest
        versions should be returned in output
        :return: list of artifacts. Each artifact is represented as dict of
        values.
        """
        session = api.get_session()
        return api.get_all(context=context, session=session, filters=filters,
                           marker=marker, limit=limit, sort=sort,
                           latest=latest)

    @retry(retry_on_exception=_retry_on_connection_error, wait_fixed=1000,
           stop_max_attempt_number=20)
    def count_artifact_number(self, context, type_name=None):
        """Count the number of artifacts for the tenant.

        :param context: user context
        :param type_name: name of specific artifact type to count artifacts.
         If None count artifacts of all types.
        :return: number of artifacts for given tenant
        """
        session = api.get_session()
        return api.count_artifact_number(context, session, type_name)

    @retry(retry_on_exception=_retry_on_connection_error, wait_fixed=1000,
           stop_max_attempt_number=20)
    def calculate_uploaded_data(self, context, type_name=None):
        """Calculate the amount of uploaded data for tenant.

        :param context: user context
        :param type_name: name of specific artifact type to calculate data.
         If None calculate data of artifacts of all types.
        :return: amount of uploaded data for given user
        """
        session = api.get_session()
        return api.calculate_uploaded_data(context, session, type_name)


class ArtifactLockApi(locking.LockApiBase):
    @retry(retry_on_exception=_retry_on_connection_error, wait_fixed=1000,
           stop_max_attempt_number=20)
    def create_lock(self, context, lock_key):
        session = api.get_session()
        return api.create_lock(context, lock_key, session)

    @retry(retry_on_exception=_retry_on_connection_error, wait_fixed=1000,
           stop_max_attempt_number=20)
    def delete_lock(self, context, lock_id):
        session = api.get_session()
        api.delete_lock(context, lock_id, session)



# Copyright (c) 2014 OpenStack Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

__all__ = [
    'list_artifacts_opts'
]

import copy
import itertools

from osprofiler import opts as profiler

import glare.api.middleware.context
import glare.api.middleware.keycloak_auth
import glare.api.v1.resource
import glare.api.versions
import glare.common.config
import glare.common.wsgi
import glare.notification
import glare.objects.base
from glare.objects.meta import registry
import glare.scrubber

_artifacts_opts = [
    (None, list(itertools.chain(
        glare.api.middleware.context.context_opts,
        glare.api.v1.resource.list_configs,
        glare.api.versions.versions_opts,
        glare.common.config.common_opts,
        glare.common.wsgi.bind_opts,
        glare.common.wsgi.eventlet_opts,
        glare.common.wsgi.socket_opts,
        glare.notification.notifier_opts,
        glare.objects.base.global_artifact_opts,
        registry.registry_options))),
    profiler.list_opts()[0],
    ('paste_deploy', glare.common.config.paste_deploy_opts),
    ('keycloak_oidc', glare.api.middleware.keycloak_auth.keycloak_oidc_opts),
    ('scrubber',
     glare.scrubber.scrubber_opts +
     glare.scrubber.scrubber_cmd_opts +
     glare.scrubber.scrubber_cmd_cli_opts)
]

registry.ArtifactRegistry.register_all_artifacts()
for af_type in registry.ArtifactRegistry.obj_classes().values():
    _artifacts_opts.append(
        ('artifact_type:' + af_type[0].get_type_name(),
         af_type[0].list_artifact_type_opts()))


def list_artifacts_opts():
    """Return a list of oslo_config options available in Glare"""
    return [(g, copy.deepcopy(o)) for g, o in _artifacts_opts]



# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from oslo_config import cfg

service_available_group = cfg.OptGroup(name='service_available',
                                       title='Available OpenStack Services')

ServiceAvailableGroup = [
    cfg.BoolOpt("glare",
                default=True,
                help="Whether or not glare is expected to be available")
]

artifacts_group = cfg.OptGroup(name="artifacts",
                               title='Glare Options')

ArtifactGroup = [
    cfg.StrOpt("catalog_type",
               default="artifact",
               help="Catalog type of Artifacts API"),
    cfg.StrOpt("endpoint_type",
               default="publicURL",
               choices=["publicURL", "adminURL", "internalURL"],
               help="The endpoint type for artifacts service")
]



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Tests for database migrations. There are "opportunistic" tests for both mysql
and postgresql in here, which allows testing against these databases in a
properly configured unit test environment.
For the opportunistic testing you need to set up a db named 'openstack_citest'
with user 'openstack_citest' and password 'openstack_citest' on localhost.
The test will then use that db and u/p combo to run the tests.
For postgres on Ubuntu this can be done with the following commands:
::
 sudo -u postgres psql
 postgres=# create user openstack_citest with createdb login password
      'openstack_citest';
 postgres=# create database openstack_citest with owner openstack_citest;
"""

import contextlib

from alembic import script
import mock
from oslo_db.sqlalchemy import utils as db_utils
from oslo_db.tests.sqlalchemy import base as test_base
from oslo_log import log as logging
import sqlalchemy
import sqlalchemy.exc

from glare.db.migration import migration
import glare.db.sqlalchemy.api
from glare.tests.unit import glare_fixtures

LOG = logging.getLogger(__name__)


@contextlib.contextmanager
def patch_with_engine(engine):
    with mock.patch.object(glare.db.sqlalchemy.api,
                           'get_engine') as patch_engine:
        patch_engine.return_value = engine
        yield


class WalkVersionsMixin(object):
    def _walk_versions(self, engine=None, alembic_cfg=None):
        # Determine latest version script from the repo, then
        # upgrade from 1 through to the latest, with no data
        # in the databases. This just checks that the schema itself
        # upgrades successfully.

        # Place the database under version control
        with patch_with_engine(engine):

            script_directory = script.ScriptDirectory.from_config(alembic_cfg)

            self.assertIsNone(self.migration_api.version(engine))

            versions = [ver for ver in script_directory.walk_revisions()]

            for version in reversed(versions):
                with glare_fixtures.BannedDBSchemaOperations():
                    self._migrate_up(engine, alembic_cfg,
                                     version.revision, with_data=True)

            for version in versions:
                with glare_fixtures.BannedDBSchemaOperations():
                    self._migrate_down(engine, alembic_cfg,
                                       version.down_revision, with_data=True)

    def _migrate_up(self, engine, config, version, with_data=False):
        """migrate up to a new version of the db.

        We allow for data insertion and post checks at every
        migration version with special _pre_upgrade_### and
        _check_### functions in the main test.
        """
        try:
            if with_data:
                data = None
                pre_upgrade = getattr(
                    self, "_pre_upgrade_%s" % version, None)
                if pre_upgrade:
                    data = pre_upgrade(engine)

            self.migration_api.upgrade(version, config=config)
            self.assertEqual(version, self.migration_api.version(engine))
            if with_data:
                check = getattr(self, "_check_%s" % version, None)
                if check:
                    check(engine, data)
        except Exception:
            LOG.error("Failed to migrate to version %(version)s on engine "
                      "%(engine)s", {'version': version, 'engine': engine})
            raise

    def _migrate_down(self, engine, config, version, with_data=False):
        try:
            self.migration_api.downgrade(version, config=config)
            if with_data:
                post_downgrade = getattr(
                    self, "_post_downgrade_%s" % version, None)
                if post_downgrade:
                    post_downgrade(engine)
        except Exception:
            LOG.error("Failed to migrate to version %(version)s on engine "
                      "%(engine)s", {'version': version, 'engine': engine})
            raise


class GlareMigrationsCheckers(object):

    def setUp(self):
        super(GlareMigrationsCheckers, self).setUp()
        self.config = migration.get_alembic_config()
        self.migration_api = migration

    def assert_table(self, engine, table_name, indices, columns):
        table = db_utils.get_table(engine, table_name)
        index_data = [(index.name, index.columns.keys()) for index in
                      table.indexes]
        column_data = [column.name for column in table.columns]
        self.assertItemsEqual(columns, column_data)
        self.assertItemsEqual(indices, index_data)

    def test_walk_versions(self):
        self._walk_versions(self.engine, self.config)

    def _pre_upgrade_001(self, engine):
        self.assertRaises(sqlalchemy.exc.NoSuchTableError,
                          db_utils.get_table, engine,
                          'glare_artifacts')
        self.assertRaises(sqlalchemy.exc.NoSuchTableError,
                          db_utils.get_table, engine,
                          'glare_artifact_tags')
        self.assertRaises(sqlalchemy.exc.NoSuchTableError,
                          db_utils.get_table, engine,
                          'glare_artifact_properties')
        self.assertRaises(sqlalchemy.exc.NoSuchTableError,
                          db_utils.get_table, engine,
                          'glare_artifact_blobs')
        self.assertRaises(sqlalchemy.exc.NoSuchTableError,
                          db_utils.get_table, engine,
                          'glare_artifact_locks')

    def _check_001(self, engine, data):
        artifacts_indices = [('ix_glare_artifact_name_and_version',
                              ['name', 'version_prefix', 'version_suffix']),
                             ('ix_glare_artifact_type',
                              ['type_name']),
                             ('ix_glare_artifact_status', ['status']),
                             ('ix_glare_artifact_visibility', ['visibility']),
                             ('ix_glare_artifact_owner', ['owner'])]
        artifacts_columns = ['id',
                             'name',
                             'type_name',
                             'version_prefix',
                             'version_suffix',
                             'version_meta',
                             'description',
                             'visibility',
                             'status',
                             'owner',
                             'created_at',
                             'updated_at',
                             'activated_at']
        self.assert_table(engine, 'glare_artifacts', artifacts_indices,
                          artifacts_columns)

        tags_indices = [('ix_glare_artifact_tags_artifact_id',
                         ['artifact_id']),
                        ('ix_glare_artifact_tags_artifact_id_tag_value',
                         ['artifact_id',
                          'value'])]
        tags_columns = ['id',
                        'artifact_id',
                        'value']
        self.assert_table(engine, 'glare_artifact_tags', tags_indices,
                          tags_columns)

        prop_indices = [
            ('ix_glare_artifact_properties_artifact_id',
             ['artifact_id']),
            ('ix_glare_artifact_properties_name', ['name'])]
        prop_columns = ['id',
                        'artifact_id',
                        'name',
                        'string_value',
                        'int_value',
                        'numeric_value',
                        'bool_value',
                        'key_name',
                        'position']
        self.assert_table(engine, 'glare_artifact_properties', prop_indices,
                          prop_columns)

        blobs_indices = [
            ('ix_glare_artifact_blobs_artifact_id', ['artifact_id']),
            ('ix_glare_artifact_blobs_name', ['name'])]
        blobs_columns = ['id',
                         'artifact_id',
                         'size',
                         'md5',
                         'sha1',
                         'sha256',
                         'name',
                         'key_name',
                         'external',
                         'status',
                         'content_type',
                         'url']
        self.assert_table(engine, 'glare_artifact_blobs', blobs_indices,
                          blobs_columns)

        locks_indices = []
        locks_columns = ['id']
        self.assert_table(engine, 'glare_artifact_locks', locks_indices,
                          locks_columns)

    def _check_002(self, engine, data):
        locks_indices = []
        locks_columns = ['id', 'acquired_at']
        self.assert_table(engine, 'glare_artifact_locks', locks_indices,
                          locks_columns)

    def _check_003(self, engine, data):
        locks_indices = []
        locks_columns = ['id', 'data']
        self.assert_table(engine, 'glare_blob_data', locks_indices,
                          locks_columns)

    def _check_004(self, engine, data):
        quota_indices = []
        quota_columns = ['project_id',
                         'quota_name',
                         'quota_value']
        self.assert_table(engine, 'glare_quotas', quota_indices,
                          quota_columns)


class TestMigrationsMySQL(GlareMigrationsCheckers,
                          WalkVersionsMixin,
                          test_base.MySQLOpportunisticTestCase):
    pass


class TestMigrationsPostgreSQL(GlareMigrationsCheckers,
                               WalkVersionsMixin,
                               test_base.PostgreSQLOpportunisticTestCase):
    pass


class TestMigrationsSqlite(GlareMigrationsCheckers,
                           WalkVersionsMixin,
                           test_base.DbTestCase,):
    pass



# Copyright 2017 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import sys
import time

from oslo_serialization import jsonutils
from six.moves import range

from glare.tests import functional
from glare.tests.functional import base
from glare.tests.utils import execute


class TestScrubber(base.TestArtifact):

    """Test that delayed_delete works and the scrubber deletes"""

    def setUp(self):
        functional.FunctionalTest.setUp(self)

        self.include_scrubber = True
        self.set_user('user1')
        self.glare_server.deployment_flavor = 'noauth'

        self.glare_server.enabled_artifact_types = ','.join(
            self.enabled_types)
        self.glare_server.custom_artifact_types_modules = (
            'glare.tests.sample_artifact')

    def _create_sample_artifact(self):
        art = self.create_artifact({'name': 'test_art',
                                    'version': '1.0'})

        url = '/sample_artifact/%s' % art['id']
        headers = {'Content-Type': 'application/octet-stream'}

        # upload data to blob
        self.put(url=url + '/small_blob', data='aaaaaa',
                 headers=headers)

        # upload a couple of blobs to dict_of_blobs
        self.put(url + '/dict_of_blobs/blob1', data='bbbb',
                 headers=headers)
        self.put(url + '/dict_of_blobs/blob2', data='cccc',
                 headers=headers)

        # add external location
        body = jsonutils.dumps(
            {'url': 'https://www.apache.org/licenses/LICENSE-2.0.txt',
             'md5': "fake", 'sha1': "fake_sha", "sha256": "fake_sha256"})
        headers = {'Content-Type':
                   'application/vnd+openstack.glare-custom-location+json'}
        self.put(url=url + '/blob', data=body, status=200,
                 headers=headers)

        return url

    def test_scrubber_delayed_delete(self):
        """
        Test that artifacts don't get deleted immediately and that the scrubber
        scrubs them.
        """
        self.start_servers(delayed_delete=True, daemon=True,
                           **self.__dict__.copy())

        url = self._create_sample_artifact()

        # create another artifact
        art2 = self.create_artifact({'name': 'test_art', 'version': '2.0'})

        # delete sample artifact
        self.delete(url=url)
        art = self.get(url)
        self.assertEqual('deleted', art['status'])

        self.wait_for_scrub(url)

        # check that the second artifact wasn't removed
        art = self.get('/sample_artifact/%s' % art2['id'])
        self.assertEqual('drafted', art['status'])

    def test_scrubber_app(self):
        """
        Test that the scrubber script runs successfully when not in
        daemon mode.
        """
        self.start_servers(delayed_delete=True,
                           **self.__dict__.copy())

        url = self._create_sample_artifact()

        # wait for the scrub time on the artifacts to pass
        time.sleep(self.scrubber_daemon.scrub_time)

        # create another artifact
        art2 = self.create_artifact({'name': 'test_art', 'version': '2.0'})

        # delete sample artifact
        self.delete(url=url)
        art = self.get(url)
        self.assertEqual('deleted', art['status'])

        # scrub artifacts and make sure they are deleted
        exe_cmd = "%s -m glare.cmd.scrubber" % sys.executable
        cmd = ("%s --config-file %s" %
               (exe_cmd, self.scrubber_daemon.conf_file_name))
        exitcode, out, err = execute(cmd, raise_error=False)
        self.assertEqual(0, exitcode)

        self.wait_for_scrub(url)

        # check that the second artifact wasn't removed
        art = self.get('/sample_artifact/%s' % art2['id'])
        self.assertEqual('drafted', art['status'])

    def wait_for_scrub(self, url):
        """
        The build servers sometimes take longer than 15 seconds
        to scrub. Give it up to 5 min, checking every 5 seconds.
        When/if it flips to deleted, bail immediately.
        """
        wait_for = 300    # seconds
        check_every = 5  # seconds
        for _ in range(wait_for // check_every):
            time.sleep(check_every)
            try:
                self.get(url, status=404)
                return
            except Exception:
                pass
        else:
            self.fail("Artifact wasn't scrubbed")



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_versionedobjects import fields

from glare.objects import base
from glare.objects.meta import fields as glare_fields
from glare.objects.meta import validators
from glare.objects.meta import wrappers

Field = wrappers.Field.init
Blob = wrappers.BlobField.init
List = wrappers.ListField.init
Dict = wrappers.DictField.init


class MuranoPackage(base.BaseArtifact):

    fields = {
        'package': Blob(required_on_activate=False,
                        description="Murano Package binary.",
                        max_blob_size=104857600),
        'type': Field(fields.StringField,
                      validators=[validators.AllowedValues(
                          ['Application', 'Library'])],
                      default='Application',
                      description="Package type."),
        'display_name': Field(fields.StringField, mutable=True,
                              description="Package name in human-readable "
                                          "format."),
        'categories': List(fields.String, mutable=True,
                           description="List of categories specified "
                                       "for the package."),
        'class_definitions': List(fields.String,
                                  validators=[validators.Unique()],
                                  description="List of class definitions in "
                                              "the package."),
        'inherits': Dict(fields.String),
        'keywords': List(fields.String, mutable=True),
        'dependencies': List(glare_fields.LinkFieldType,
                             required_on_activate=False,
                             description="List of package dependencies for "
                                         "this package."),
    }

    @classmethod
    def get_type_name(cls):
        return "murano_packages"



# Copyright 2010-2011 OpenStack Foundation
# Copyright 2014 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
import os
import socket

import eventlet.patcher
import fixtures
import mock
from oslo_concurrency import processutils
from oslo_serialization import jsonutils
import routes
import six
from six.moves import http_client as http
import webob

from glare.api.v1 import router
from glare.common import exception
from glare.common import wsgi
from glare import i18n
from glare.tests.unit import base


class RequestTest(base.BaseTestCase):

    def test_content_range(self):
        request = wsgi.Request.blank('/tests/123')
        request.headers["Content-Range"] = 'bytes 10-99/*'
        range_ = request.get_content_range()
        self.assertEqual(10, range_.start)
        self.assertEqual(100, range_.stop)  # non-inclusive
        self.assertIsNone(range_.length)

    def test_content_range_invalid(self):
        request = wsgi.Request.blank('/tests/123')
        request.headers["Content-Range"] = 'bytes=0-99'
        self.assertRaises(webob.exc.HTTPBadRequest,
                          request.get_content_range)

    def test_language_accept_default(self):
        request = wsgi.Request.blank('/tests/123')
        request.headers["Accept-Language"] = "zz-ZZ,zz;q=0.8"
        result = request.best_match_language()
        self.assertIsNone(result)

    def test_language_accept_none(self):
        request = wsgi.Request.blank('/tests/123')
        result = request.best_match_language()
        self.assertIsNone(result)

    def test_best_match_language_expected(self):
        # If Accept-Language is a supported language, best_match_language()
        # returns it.
        with mock.patch('babel.localedata.locale_identifiers',
                        return_value=['en']):
            req = wsgi.Request.blank('/', headers={'Accept-Language': 'en'})
        self.assertEqual('en_US', req.best_match_language())

    def test_request_match_language_unexpected(self):
        # If Accept-Language is a language we do not support,
        # best_match_language() returns None.
        with mock.patch('babel.localedata.locale_identifiers',
                        return_value=['en']):
            req = wsgi.Request.blank(
                '/', headers={'Accept-Language': 'Klingon'})
        self.assertIsNone(req.best_match_language())

    @mock.patch.object(webob.acceptparse.AcceptLanguage, 'best_match')
    def test_best_match_language_unknown(self, mock_best_match):
        # Test that we are actually invoking language negotiation by webop
        request = wsgi.Request.blank('/')
        accepted = 'unknown-lang'
        request.headers = {'Accept-Language': accepted}

        mock_best_match.return_value = None

        self.assertIsNone(request.best_match_language())

        # If Accept-Language is missing or empty, match should be None
        request.headers = {'Accept-Language': ''}
        self.assertIsNone(request.best_match_language())
        request.headers.pop('Accept-Language')
        self.assertIsNone(request.best_match_language())

    def test_http_error_response_codes(self):
        """Makes sure v1 unallowed methods return 405"""
        unallowed_methods = [
            ('/schemas', ['PUT', 'DELETE', 'HEAD', 'PATCH', 'POST']),
            ('/schemas/type_name', ['PUT', 'DELETE', 'HEAD', 'PATCH', 'POST']),
            ('/artifacts/type_name', ['PUT', 'DELETE', 'HEAD', 'PATCH']),
            ('/artifacts/type_name/artifact_id', ['PUT', 'HEAD', 'POST']),
            ('/artifacts/type_name/artifact_id/blob)name',
             ['HEAD', 'PATCH', 'POST']),
        ]
        api = router.API(routes.Mapper())
        for uri, methods in unallowed_methods:
            for method in methods:
                req = webob.Request.blank(uri)
                req.method = method
                res = req.get_response(api)
                self.assertEqual(http.METHOD_NOT_ALLOWED, res.status_int)

        # Makes sure not implemented methods return 405
        req = webob.Request.blank('/schemas/image')
        req.method = 'NonexistentMethod'
        res = req.get_response(api)
        self.assertEqual(http.METHOD_NOT_ALLOWED, res.status_int)


class ResourceTest(base.BaseTestCase):

    def test_get_action_args(self):
        env = {
            'wsgiorg.routing_args': [
                None,
                {
                    'controller': None,
                    'format': None,
                    'action': 'update',
                    'id': 12,
                },
            ],
        }

        expected = {'action': 'update', 'id': 12}
        actual = wsgi.Resource(None, None, None).get_action_args(env)

        self.assertEqual(expected, actual)

    def test_get_action_args_invalid_index(self):
        env = {'wsgiorg.routing_args': []}
        expected = {}
        actual = wsgi.Resource(None, None, None).get_action_args(env)
        self.assertEqual(expected, actual)

    def test_get_action_args_del_controller_error(self):
        actions = {'format': None,
                   'action': 'update',
                   'id': 12}
        env = {'wsgiorg.routing_args': [None, actions]}
        expected = {'action': 'update', 'id': 12}
        actual = wsgi.Resource(None, None, None).get_action_args(env)
        self.assertEqual(expected, actual)

    def test_get_action_args_del_format_error(self):
        actions = {'action': 'update', 'id': 12}
        env = {'wsgiorg.routing_args': [None, actions]}
        expected = {'action': 'update', 'id': 12}
        actual = wsgi.Resource(None, None, None).get_action_args(env)
        self.assertEqual(expected, actual)

    def test_dispatch(self):
        class Controller(object):
            def index(self, shirt, pants=None):
                return (shirt, pants)

        resource = wsgi.Resource(None, None, None)
        actual = resource.dispatch(Controller(), 'index', 'on', pants='off')
        expected = ('on', 'off')
        self.assertEqual(expected, actual)

    def test_dispatch_default(self):
        class Controller(object):
            def default(self, shirt, pants=None):
                return (shirt, pants)

        resource = wsgi.Resource(None, None, None)
        actual = resource.dispatch(Controller(), 'index', 'on', pants='off')
        expected = ('on', 'off')
        self.assertEqual(expected, actual)

    def test_dispatch_no_default(self):
        class Controller(object):
            def show(self, shirt, pants=None):
                return (shirt, pants)

        resource = wsgi.Resource(None, None, None)
        self.assertRaises(AttributeError, resource.dispatch, Controller(),
                          'index', 'on', pants='off')

    def test_call(self):
        class FakeController(object):
            def index(self, shirt, pants=None):
                return shirt, pants

        resource = wsgi.Resource(FakeController(), None, None)

        def dispatch(obj, *args, **kwargs):
            if isinstance(obj, wsgi.JSONRequestDeserializer):
                return []
            if isinstance(obj, wsgi.JSONResponseSerializer):
                raise webob.exc.HTTPForbidden()

        with mock.patch('glare.common.wsgi.Resource.dispatch',
                        side_effect=dispatch):
            request = wsgi.Request.blank('/')
            response = resource.__call__(request)

        self.assertIsInstance(response, webob.exc.HTTPForbidden)
        self.assertEqual(http.FORBIDDEN, response.status_code)

    def test_call_raises_exception(self):
        class FakeController(object):
            def index(self, shirt, pants=None):
                return (shirt, pants)

        resource = wsgi.Resource(FakeController(), None, None)

        with mock.patch('glare.common.wsgi.Resource.dispatch',
                        side_effect=Exception("test exception")):
            request = wsgi.Request.blank('/')
            response = resource.__call__(request)

        self.assertIsInstance(response, webob.exc.HTTPInternalServerError)
        self.assertEqual(http.INTERNAL_SERVER_ERROR, response.status_code)

    @mock.patch.object(wsgi, 'translate_exception')
    def test_resource_call_error_handle_localized(self,
                                                  mock_translate_exception):
        class Controller(object):
            def delete(self, req, identity):
                raise webob.exc.HTTPBadRequest(explanation='Not Found')

        actions = {'action': 'delete', 'identity': 12}
        env = {'wsgiorg.routing_args': [None, actions]}
        request = wsgi.Request.blank('/tests/123', environ=env)
        message_es = 'No Encontrado'

        resource = wsgi.Resource(Controller(),
                                 wsgi.JSONRequestDeserializer(),
                                 None)
        translated_exc = webob.exc.HTTPBadRequest(message_es)
        mock_translate_exception.return_value = translated_exc

        e = self.assertRaises(webob.exc.HTTPBadRequest,
                              resource, request)
        self.assertEqual(message_es, str(e))

    @mock.patch.object(webob.acceptparse.AcceptLanguage, 'best_match')
    @mock.patch.object(i18n, 'translate')
    def test_translate_exception(self, mock_translate, mock_best_match):

        mock_translate.return_value = 'No Encontrado'
        mock_best_match.return_value = 'de'

        req = wsgi.Request.blank('/tests/123')
        req.headers["Accept-Language"] = "de"

        e = webob.exc.HTTPNotFound(explanation='Not Found')
        e = wsgi.translate_exception(req, e)
        self.assertEqual('No Encontrado', e.explanation)

    def test_response_headers_encoded(self):
        # prepare environment
        for_openstack_comrades =  \
            u'\u0417\u0430 \u043e\u043f\u0435\u043d\u0441\u0442\u0435\u043a, ' \
            u'\u0442\u043e\u0432\u0430\u0440\u0438\u0449\u0438'

        class FakeController(object):
            def index(self, shirt, pants=None):
                return (shirt, pants)

        class FakeSerializer(object):
            def index(self, response, result):
                response.headers['unicode_test'] = for_openstack_comrades

        # make request
        resource = wsgi.Resource(FakeController(), None, FakeSerializer())
        actions = {'action': 'index'}
        env = {'wsgiorg.routing_args': [None, actions]}
        request = wsgi.Request.blank('/tests/123', environ=env)
        response = resource.__call__(request)

        # ensure it has been encoded correctly
        value = (response.headers['unicode_test'].decode('utf-8')
                 if six.PY2 else response.headers['unicode_test'])
        self.assertEqual(for_openstack_comrades, value)


class JSONResponseSerializerTest(base.BaseTestCase):

    def test_to_json(self):
        fixture = {"key": "value"}
        expected = b'{"key": "value"}'
        actual = wsgi.JSONResponseSerializer().to_json(fixture)
        self.assertEqual(expected, actual)

    def test_to_json_with_date_format_value(self):
        fixture = {"date": datetime.datetime(1901, 3, 8, 2)}
        expected = b'{"date": "1901-03-08T02:00:00.000000"}'
        actual = wsgi.JSONResponseSerializer().to_json(fixture)
        self.assertEqual(expected, actual)

    def test_to_json_with_more_deep_format(self):
        fixture = {"is_public": True, "name": [{"name1": "test"}]}
        expected = {"is_public": True, "name": [{"name1": "test"}]}
        actual = wsgi.JSONResponseSerializer().to_json(fixture)
        actual = jsonutils.loads(actual)
        for k in expected:
            self.assertEqual(expected[k], actual[k])

    def test_to_json_with_set(self):
        fixture = set(["foo"])
        expected = b'["foo"]'
        actual = wsgi.JSONResponseSerializer().to_json(fixture)
        self.assertEqual(expected, actual)

    def test_default(self):
        fixture = {"key": "value"}
        response = webob.Response()
        wsgi.JSONResponseSerializer().default(response, fixture)
        self.assertEqual(http.OK, response.status_int)
        content_types = [h for h in response.headerlist
                         if h[0] == 'Content-Type']
        self.assertEqual(1, len(content_types))
        self.assertEqual('application/json', response.content_type)
        self.assertEqual(b'{"key": "value"}', response.body)


class JSONRequestDeserializerTest(base.BaseTestCase):

    def test_has_body_no_content_length(self):
        request = wsgi.Request.blank('/')
        request.method = 'POST'
        request.body = b'asdf'
        request.headers.pop('Content-Length')
        self.assertFalse(wsgi.JSONRequestDeserializer().has_body(request))

    def test_has_body_zero_content_length(self):
        request = wsgi.Request.blank('/')
        request.method = 'POST'
        request.body = b'asdf'
        request.headers['Content-Length'] = 0
        self.assertFalse(wsgi.JSONRequestDeserializer().has_body(request))

    def test_has_body_has_content_length(self):
        request = wsgi.Request.blank('/')
        request.method = 'POST'
        request.body = b'asdf'
        self.assertIn('Content-Length', request.headers)
        self.assertTrue(wsgi.JSONRequestDeserializer().has_body(request))

    def test_no_body_no_content_length(self):
        request = wsgi.Request.blank('/')
        self.assertFalse(wsgi.JSONRequestDeserializer().has_body(request))

    def test_from_json(self):
        fixture = '{"key": "value"}'
        expected = {"key": "value"}
        actual = wsgi.JSONRequestDeserializer().from_json(fixture)
        self.assertEqual(expected, actual)

    def test_from_json_malformed(self):
        fixture = 'kjasdklfjsklajf'
        self.assertRaises(webob.exc.HTTPBadRequest,
                          wsgi.JSONRequestDeserializer().from_json, fixture)

    def test_default_no_body(self):
        request = wsgi.Request.blank('/')
        actual = wsgi.JSONRequestDeserializer().default(request)
        expected = {}
        self.assertEqual(expected, actual)

    def test_default_with_body(self):
        request = wsgi.Request.blank('/')
        request.method = 'POST'
        request.body = b'{"key": "value"}'
        actual = wsgi.JSONRequestDeserializer().default(request)
        expected = {"body": {"key": "value"}}
        self.assertEqual(expected, actual)

    def test_has_body_has_transfer_encoding(self):
        self.assertTrue(self._check_transfer_encoding(
                        transfer_encoding='chunked'))

    def test_has_body_multiple_transfer_encoding(self):
        self.assertTrue(self._check_transfer_encoding(
                        transfer_encoding='chunked, gzip'))

    def test_has_body_invalid_transfer_encoding(self):
        self.assertFalse(self._check_transfer_encoding(
                         transfer_encoding='invalid', content_length=0))

    def test_has_body_invalid_transfer_encoding_no_content_len_and_body(self):
        self.assertFalse(self._check_transfer_encoding(
                         transfer_encoding='invalid', include_body=False))

    def test_has_body_invalid_transfer_encoding_no_content_len_but_body(self):
        self.assertTrue(self._check_transfer_encoding(
                        transfer_encoding='invalid', include_body=True))

    def test_has_body_invalid_transfer_encoding_with_content_length(self):
        self.assertTrue(self._check_transfer_encoding(
                        transfer_encoding='invalid', content_length=5))

    def test_has_body_valid_transfer_encoding_with_content_length(self):
        self.assertTrue(self._check_transfer_encoding(
                        transfer_encoding='chunked', content_length=1))

    def test_has_body_valid_transfer_encoding_without_content_length(self):
        self.assertTrue(self._check_transfer_encoding(
                        transfer_encoding='chunked'))

    def _check_transfer_encoding(self, transfer_encoding=None,
                                 content_length=None, include_body=True):
        request = wsgi.Request.blank('/')
        request.method = 'POST'
        if include_body:
            request.body = b'fake_body'
        request.headers['transfer-encoding'] = transfer_encoding
        if content_length is not None:
            request.headers['content-length'] = content_length

        return wsgi.JSONRequestDeserializer().has_body(request)

    def test_get_bind_addr_default_value(self):
        expected = ('0.0.0.0', '123456')
        actual = wsgi.get_bind_addr(default_port="123456")
        self.assertEqual(expected, actual)


class ServerTest(base.BaseTestCase):
    def test_create_pool(self):
        """Ensure the wsgi thread pool is an eventlet.greenpool.GreenPool."""
        actual = wsgi.Server(threads=1).create_pool()
        self.assertIsInstance(actual, eventlet.greenpool.GreenPool)

    @mock.patch.object(wsgi.Server, 'configure_socket')
    def test_http_keepalive(self, mock_configure_socket):
        self.config(http_keepalive=False)
        self.config(workers=None)

        server = wsgi.Server(threads=1)
        server.sock = 'fake_socket'
        # mocking eventlet.wsgi server method to check it is called with
        # configured 'http_keepalive' value.
        with mock.patch.object(eventlet.wsgi,
                               'server') as mock_server:
            fake_application = "fake-application"
            server.start(fake_application, 0)
            server.wait()
            mock_server.assert_called_once_with('fake_socket',
                                                fake_application,
                                                log=server._logger,
                                                debug=False,
                                                custom_pool=server.pool,
                                                keepalive=False,
                                                socket_timeout=900)

    def test_number_of_workers(self):
        """Ensure the default number of workers matches num cpus."""
        def pid():
            i = 1
            while True:
                i += 1
                yield i

        with mock.patch.object(os, 'fork') as mock_fork:
            mock_fork.side_effect = pid
            server = wsgi.Server()
            server.configure = mock.Mock()
            fake_application = "fake-application"
            server.start(fake_application, None)
            self.assertEqual(processutils.get_worker_count(),
                             len(server.children))

    def test_set_eventlet_hub_exception(self):
        with mock.patch('eventlet.hubs.use_hub', side_effect=Exception):
            self.assertRaises(exception.WorkerCreationFailure,
                              wsgi.set_eventlet_hub)


class GetSocketTestCase(base.BaseTestCase):

    def setUp(self):
        super(GetSocketTestCase, self).setUp()
        self.useFixture(fixtures.MonkeyPatch(
            "glare.common.wsgi.get_bind_addr",
            lambda x: ('192.168.0.13', 1234)))
        addr_info_list = [(2, 1, 6, '', ('192.168.0.13', 80)),
                          (2, 2, 17, '', ('192.168.0.13', 80)),
                          (2, 3, 0, '', ('192.168.0.13', 80))]
        self.useFixture(fixtures.MonkeyPatch(
            "glare.common.wsgi.socket.getaddrinfo",
            lambda *x: addr_info_list))
        self.useFixture(fixtures.MonkeyPatch(
            "glare.common.wsgi.time.time",
            mock.Mock(side_effect=[0, 1, 5, 10, 20, 35])))
        self.useFixture(fixtures.MonkeyPatch(
            "glare.common.wsgi.utils.validate_key_cert",
            lambda *x: None))
        wsgi.CONF.cert_file = '/etc/ssl/cert'
        wsgi.CONF.key_file = '/etc/ssl/key'
        wsgi.CONF.ca_file = '/etc/ssl/ca_cert'
        wsgi.CONF.tcp_keepidle = 600

    def test_correct_configure_socket(self):
        mock_socket = mock.Mock()
        self.useFixture(fixtures.MonkeyPatch(
            'glare.common.wsgi.ssl.wrap_socket',
            mock_socket))
        self.useFixture(fixtures.MonkeyPatch(
            'glare.common.wsgi.eventlet.listen',
            lambda *x, **y: mock_socket))
        server = wsgi.Server()
        server.default_port = 1234
        server.configure_socket()
        self.assertIn(mock.call.setsockopt(
            socket.SOL_SOCKET,
            socket.SO_REUSEADDR,
            1), mock_socket.mock_calls)
        self.assertIn(mock.call.setsockopt(
            socket.SOL_SOCKET,
            socket.SO_KEEPALIVE,
            1), mock_socket.mock_calls)
        if hasattr(socket, 'TCP_KEEPIDLE'):
            self.assertIn(mock.call().setsockopt(
                socket.IPPROTO_TCP,
                socket.TCP_KEEPIDLE,
                wsgi.CONF.tcp_keepidle), mock_socket.mock_calls)

    def test_get_socket_without_all_ssl_reqs(self):
        wsgi.CONF.key_file = None
        self.assertRaises(RuntimeError, wsgi.get_socket, 1234)

    def test_get_socket_with_bind_problems(self):
        self.useFixture(fixtures.MonkeyPatch(
            'glare.common.wsgi.eventlet.listen',
            mock.Mock(side_effect=(
                [wsgi.socket.error(socket.errno.EADDRINUSE)] * 3 + [None]))))
        self.useFixture(fixtures.MonkeyPatch(
            'glare.common.wsgi.ssl.wrap_socket',
            lambda *x, **y: None))

        self.assertRaises(RuntimeError, wsgi.get_socket, 1234)

    def test_get_socket_with_unexpected_socket_errno(self):
        self.useFixture(fixtures.MonkeyPatch(
            'glare.common.wsgi.eventlet.listen',
            mock.Mock(side_effect=wsgi.socket.error(socket.errno.ENOMEM))))
        self.useFixture(fixtures.MonkeyPatch(
            'glare.common.wsgi.ssl.wrap_socket',
            lambda *x, **y: None))
        self.assertRaises(wsgi.socket.error, wsgi.get_socket, 1234)



# Copyright 2017 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import io
import zipfile

from glare.common import exception
from glare.objects import base
from glare.objects.meta import file_utils
from glare.objects.meta import wrappers

Blob = wrappers.BlobField.init
Folder = wrappers.FolderField.init


class Unpacker(base.BaseArtifact):
    MAX_BLOB_SIZE = 100000

    fields = {
        'zip': Blob(description="Original zipped data.",
                    required_on_activate=False),
        'content': Folder(system=True, required_on_activate=False),
    }

    @classmethod
    def get_type_name(cls):
        return "unpacking_artifact"

    @classmethod
    def pre_upload_hook(cls, context, af, field_name, blob_key, fd):
        flobj = io.BytesIO(fd.read(cls.MAX_BLOB_SIZE))

        # Raise exception if something left in the stream
        if fd.read(1):
            msg = ("The file you are trying to upload is too big. "
                   "The system upper limit is %s.") % cls.MAX_BLOB_SIZE
            raise exception.RequestEntityTooLarge(msg)

        zip_ref = zipfile.ZipFile(flobj, 'r')

        file_utils.unpack_zip_archive_to_artifact_folder(
            context, af, zip_ref, 'content')

        flobj.seek(0)
        return flobj



# Copyright (c) 2015 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import operator

import semantic_version
from sqlalchemy.orm.properties import CompositeProperty
from sqlalchemy import sql

from glare.common import exception
from glare.i18n import _

MAX_COMPONENT_LENGTH = pow(2, 16) - 1
MAX_NUMERIC_PRERELEASE_LENGTH = 6


class DBVersion(object):
    def __init__(self, components_long, prerelease, build):
        """Creates a DBVersion object out of 3 component fields. This initializer
        is supposed to be called from SQLAlchemy if 3 database columns are
        mapped to this composite field.

        :param components_long: a 64-bit long value, containing numeric
         components of the version
        :param prerelease: a prerelease label of the version, optionally
         preformatted with leading zeroes in numeric-only parts of the label
        :param build: a build label of the version
        """
        version_string = '%s.%s.%s' % _long_to_components(components_long)
        if prerelease:
            version_string += '-' + _strip_leading_zeroes_from_prerelease(
                prerelease)

        if build:
            version_string += '+' + build
        self.version = semantic_version.Version(version_string)

    def __repr__(self):
        return str(self.version)

    def __eq__(self, other):
        return (isinstance(other, DBVersion) and
                other.version == self.version)

    def __ne__(self, other):
        return (not isinstance(other, DBVersion)
                or self.version != other.version)

    def __composite_values__(self):
        long_version = _version_to_long(self.version)
        prerelease = _add_leading_zeroes_to_prerelease(self.version.prerelease)
        build = '.'.join(self.version.build) if self.version.build else None
        return long_version, prerelease, build


def parse(version_string):
    version = semantic_version.Version.coerce(version_string)
    return DBVersion(_version_to_long(version),
                     '.'.join(version.prerelease),
                     '.'.join(version.build))


def _check_limit(value):
    if value > MAX_COMPONENT_LENGTH:
        message = _("Version component is too "
                    "large (%d max)") % MAX_COMPONENT_LENGTH
        raise exception.InvalidVersion(message)


def _version_to_long(version):
    """Converts the numeric part of the semver version into the 64-bit long value
    using the following logic:

    * major version is stored in first 16 bits of the value
    * minor version is stored in next 16 bits
    * patch version is stored in following 16 bits
    * next 2 bits are used to store the flag: if the version has pre-release
      label then these bits are 00, otherwise they are 11. Intermediate values
      of the flag (01 and 10) are reserved for future usage.
    * last 14 bits of the value are reserved for future usage

    The numeric components of version are checked so their value does not
    exceed 16 bits.

    :param version: a semantic_version.Version object
    """
    _check_limit(version.major)
    _check_limit(version.minor)
    _check_limit(version.patch)
    major = version.major << 48
    minor = version.minor << 32
    patch = version.patch << 16
    flag = 0 if version.prerelease else 2
    flag <<= 14
    return major | minor | patch | flag


def _long_to_components(value):
    major = value >> 48
    minor = (value - (major << 48)) >> 32
    patch = (value - (major << 48) - (minor << 32)) >> 16
    return str(major), str(minor), str(patch)


def _add_leading_zeroes_to_prerelease(label_tuple):
    if label_tuple is None:
        return None
    res = []
    for component in label_tuple:
        if component.isdigit():
            if len(component) > MAX_NUMERIC_PRERELEASE_LENGTH:
                message = _("Prerelease numeric component is too large "
                            "(%d characters "
                            "max)") % MAX_NUMERIC_PRERELEASE_LENGTH
                raise exception.InvalidVersion(message)
            res.append(component.rjust(MAX_NUMERIC_PRERELEASE_LENGTH, '0'))
        else:
            res.append(component)
    return '.'.join(res)


def _strip_leading_zeroes_from_prerelease(string_value):
    res = []
    for component in string_value.split('.'):
        if component.isdigit():
            val = component.lstrip('0')
            if len(val) == 0:  # Corner case: when the component is just '0'
                val = '0'  # it will be stripped completely, so restore it
            res.append(val)
        else:
            res.append(component)
    return '.'.join(res)

strict_op_map = {
    operator.ge: operator.gt,
    operator.le: operator.lt
}


class VersionComparator(CompositeProperty.Comparator):
    def _get_comparison(self, values, op):
        columns = self.__clause_element__().clauses
        if op in strict_op_map:
            stricter_op = strict_op_map[op]
        else:
            stricter_op = op

        return sql.or_(stricter_op(columns[0], values[0]),
                       sql.and_(columns[0] == values[0],
                                op(columns[1], values[1])))

    def __gt__(self, other):
        return self._get_comparison(other.__composite_values__(), operator.gt)

    def __ge__(self, other):
        return self._get_comparison(other.__composite_values__(), operator.ge)

    def __lt__(self, other):
        return self._get_comparison(other.__composite_values__(), operator.lt)

    def __le__(self, other):
        return self._get_comparison(other.__composite_values__(), operator.le)



# Copyright 2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from oslo_versionedobjects import fields

from glare.objects import base
from glare.objects.meta import validators
from glare.objects.meta import wrappers

Field = wrappers.Field.init
Blob = wrappers.BlobField.init


class Image(base.BaseArtifact):

    fields = {
        'container_format': Field(fields.StringField,
                                  validators=[validators.AllowedValues(
                                      ['ami', 'ari', 'aki', 'bare',
                                       'ovf', 'ova', 'docker'])],
                                  description="Image container format."),
        'disk_format': Field(fields.StringField,
                             validators=[validators.AllowedValues(
                                 ['ami', 'ari', 'aki', 'vhd', 'vhdx',
                                  'vmdk', 'raw', 'qcow2', 'vdi', 'iso'])],
                             description="Image disk format."),
        'min_ram': Field(fields.IntegerField, required_on_activate=False,
                         validators=[validators.MinNumberSize(0)],
                         description="Minimal RAM required to boot image."),
        'min_disk': Field(fields.IntegerField, required_on_activate=False,
                          validators=[validators.MinNumberSize(0)],
                          description="Minimal disk space "
                                      "required to boot image."),
        'image': Blob(max_blob_size=1073741824000,
                      required_on_activate=False,
                      description="Image binary."),
        'kernel_id': Field(fields.StringField,
                           required_on_activate=False,
                           validators=[validators.UUID()],
                           description="ID of image stored in Glare that "
                                       "should be used as the kernel when "
                                       "booting an AMI-style image."),
        'ramdisk_id': Field(fields.StringField,
                            required_on_activate=False,
                            validators=[validators.UUID()],
                            description="ID of image stored in Glare that "
                                        "should be used as the ramdisk when "
                                        "booting an AMI-style image."),
        'instance_uuid': Field(fields.StringField,
                               required_on_activate=False,
                               description="Metadata which can be used to "
                                           "record which instance this image "
                                           "is associated with. "
                                           "(Informational only, does not "
                                           "create an instance snapshot.)"),
        'architecture': Field(fields.StringField,
                              required_on_activate=False,
                              description="Operating system architecture as "
                                          "specified in http://docs.openstack."
                                          "org/trunk/openstack-compute/admin/"
                                          "content/adding-images.html"),
        'os_distro': Field(fields.StringField,
                           required_on_activate=False,
                           description="Common name of operating system "
                                       "distribution as specified in "
                                       "http://docs.openstack.org/trunk/"
                                       "openstack-compute/admin/content/"
                                       "adding-images.html"),
        'os_version': Field(fields.StringField,
                            required_on_activate=False,
                            description="Operating system version as "
                                        "specified by the distributor"),
    }

    @classmethod
    def get_type_name(cls):
        return "images"



# Copyright 2017 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import tempfile

from oslo_config import cfg
from oslo_log import log as logging
from oslo_versionedobjects import fields

from glare.objects import base
from glare.objects.meta import wrappers

Field = wrappers.Field.init
Dict = wrappers.DictField.init
List = wrappers.ListField.init
Blob = wrappers.BlobField.init
Folder = wrappers.FolderField.init

CONF = cfg.CONF
LOG = logging.getLogger(__name__)


class HookChecker(base.BaseArtifact):
    fields = {
        'temp_dir': Field(
            fields.StringField,
            required_on_activate=False,
            mutable=True),
        'temp_file_path_create': Field(
            fields.StringField,
            required_on_activate=False,
            mutable=True),
        'temp_file_path_update': Field(
            fields.StringField,
            required_on_activate=False,
            mutable=True),
        'temp_file_path_activate': Field(
            fields.StringField,
            required_on_activate=False,
            mutable=True),
        'temp_file_path_reactivate': Field(
            fields.StringField,
            required_on_activate=False,
            mutable=True),
        'temp_file_path_deactivate': Field(
            fields.StringField,
            required_on_activate=False,
            mutable=True),
        'temp_file_path_publish': Field(
            fields.StringField,
            required_on_activate=False,
            mutable=True),
        'blob': Blob(
            required_on_activate=False,
            mutable=True)
    }

    artifact_type_opts = [
        cfg.StrOpt('temp_file_path')
    ]

    @classmethod
    def get_type_name(cls):
        return "hooks_artifact"

    @classmethod
    def pre_create_hook(cls, context, af):
        # create a temporary file and set the path to artifact field
        __, af.temp_file_path_create = tempfile.mkstemp(dir=af.temp_dir)
        with open(af.temp_file_path_create, 'w') as f:
            f.write('pre_create_hook was called\n')

    @classmethod
    def post_create_hook(cls, context, af):
        with open(af.temp_file_path_create, 'a') as f:
            f.write('post_create_hook was called\n')

    @classmethod
    def pre_update_hook(cls, context, af):
        # create a temporary file and set the path to artifact field
        __, af.temp_file_path_update = tempfile.mkstemp(dir=af.temp_dir)
        with open(af.temp_file_path_update, 'w') as f:
            f.write('pre_update_hook was called\n')

    @classmethod
    def post_update_hook(cls, context, af):
        with open(af.temp_file_path_update, 'a') as f:
            f.write('post_update_hook was called\n')

    @classmethod
    def pre_activate_hook(cls, context, af):
        # create a temporary file and set the path to artifact field
        __, af.temp_file_path_activate = tempfile.mkstemp(dir=af.temp_dir)
        with open(af.temp_file_path_activate, 'w') as f:
            f.write('pre_activate_hook was called\n')

    @classmethod
    def post_activate_hook(cls, context, af):
        with open(af.temp_file_path_activate, 'a') as f:
            f.write('post_activate_hook was called\n')

    @classmethod
    def pre_publish_hook(cls, context, af):
        # create a temporary file and set the path to artifact field
        __, af.temp_file_path_publish = tempfile.mkstemp(dir=af.temp_dir)
        with open(af.temp_file_path_publish, 'w') as f:
            f.write('pre_publish_hook was called\n')

    @classmethod
    def post_publish_hook(cls, context, af):
        with open(af.temp_file_path_publish, 'a') as f:
            f.write('post_publish_hook was called\n')

    @classmethod
    def pre_deactivate_hook(cls, context, af):
        # create a temporary file and set the path to artifact field
        __, af.temp_file_path_deactivate = tempfile.mkstemp(dir=af.temp_dir)
        with open(af.temp_file_path_deactivate, 'w') as f:
            f.write('pre_deactivate_hook was called\n')

    @classmethod
    def post_deactivate_hook(cls, context, af):
        with open(af.temp_file_path_deactivate, 'a') as f:
            f.write('post_deactivate_hook was called\n')

    @classmethod
    def pre_reactivate_hook(cls, context, af):
        # create a temporary file and set the path to artifact field
        __, af.temp_file_path_reactivate = tempfile.mkstemp(dir=af.temp_dir)
        with open(af.temp_file_path_reactivate, 'w') as f:
            f.write('pre_reactivate_hook was called\n')

    @classmethod
    def post_reactivate_hook(cls, context, af):
        with open(af.temp_file_path_reactivate, 'a') as f:
            f.write('post_reactivate_hook was called\n')

    @classmethod
    def pre_upload_hook(cls, context, af, field_name, blob_key, fd):
        # create a temporary file and set the path to artifact field
        file_path = getattr(
            CONF, 'artifact_type:hooks_artifact').temp_file_path
        if file_path:
            with open(file_path, 'w') as f:
                f.write('pre_upload_hook was called\n')
        return fd

    @classmethod
    def post_upload_hook(cls, context, af, field_name, blob_key):
        file_path = getattr(
            CONF, 'artifact_type:hooks_artifact').temp_file_path
        if file_path:
            with open(file_path, 'a') as f:
                f.write('post_upload_hook was called\n')

    @classmethod
    def pre_add_location_hook(
            cls, context, af, field_name, blob_key, location):
        # create a temporary file and set the path to artifact field
        file_path = getattr(
            CONF, 'artifact_type:hooks_artifact').temp_file_path
        if file_path:
            with open(file_path, 'w') as f:
                f.write('pre_add_location_hook was called\n')

    @classmethod
    def post_add_location_hook(cls, context, af, field_name, blob_key):
        file_path = getattr(
            CONF, 'artifact_type:hooks_artifact').temp_file_path
        if file_path:
            with open(file_path, 'a') as f:
                f.write('post_add_location_hook was called\n')

    @classmethod
    def pre_download_hook(cls, context, af, field_name, blob_key):
        file_path = getattr(
            CONF, 'artifact_type:hooks_artifact').temp_file_path
        if file_path:
            with open(file_path, 'a') as f:
                f.write('pre_download_hook was called\n')

    @classmethod
    def post_download_hook(cls, context, af, field_name, blob_key, fd):
        file_path = getattr(
            CONF, 'artifact_type:hooks_artifact').temp_file_path
        if file_path:
            with open(file_path, 'a') as f:
                f.write('post_download_hook was called\n')
        return fd

    @classmethod
    def pre_delete_hook(cls, context, af):
        file_path = getattr(
            CONF, 'artifact_type:hooks_artifact').temp_file_path
        if file_path:
            with open(file_path, 'w') as f:
                f.write('pre_delete_hook was called\n')

    @classmethod
    def post_delete_hook(cls, context, af):
        file_path = getattr(
            CONF, 'artifact_type:hooks_artifact').temp_file_path
        if file_path:
            with open(file_path, 'a') as f:
                f.write('post_delete_hook was called\n')



# Copyright 2017 Nokia
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Contains additional file utils that may be useful for upload hooks."""

import os
import tempfile
import zipfile

from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import excutils
from oslo_utils import uuidutils

from glare.common import store_api
from glare.common import utils
from glare.objects.meta import fields as glare_fields

CONF = cfg.CONF
LOG = logging.getLogger(__name__)

INMEMORY_OBJECT_SIZE_LIMIT = 134217728  # 128 megabytes


def create_temporary_file(stream, suffix=''):
    """Create a temporary local file from a stream.

    :param stream: stream of bytes to be stored in a temporary file
    :param suffix: (optional) file name suffix
    """
    tfd, path = tempfile.mkstemp(suffix=suffix)
    while True:
        data = stream.read(100000)
        if data == b'':  # end of file reached
            break
        os.write(tfd, data)
    tfile = os.fdopen(tfd, "rb")
    return tfile, path


def extract_zip_to_temporary_folder(tfile):
    """Create temporary folder and extract all file contents there.

    :param tfile: zip archive to be extracted
    """
    zip_ref = zipfile.ZipFile(tfile, 'r')
    tdir = tempfile.mkdtemp()
    zip_ref.extractall(tdir)
    zip_ref.close()
    return tdir


def unpack_zip_archive_to_artifact_folder(context, af, zip_ref, folder_name):
    """Unpack zip archive to artifact folder.

    :param context: user context
    :param af: artifact object
    :param zip_ref: zip archive to be extracted
    :param folder_name: name of the artifact folder where to extract data
    """
    file_dict = {}
    blobs = []
    for name in zip_ref.namelist():
        if not name.endswith('/'):
            blob_id = uuidutils.generate_uuid()
            # create an an empty blob instance in db with 'saving' status
            blob = {'url': None, 'size': None, 'md5': None, 'sha1': None,
                    'sha256': None, 'status': 'saving', 'id': blob_id,
                    'external': False,
                    'content_type': 'application/octet-stream'}
            file_dict[name] = blob
            blobs.append((blob_id, utils.BlobIterator(zip_ref.read(name))))

    af = af.update_blob(context, af.id, folder_name, file_dict)

    default_store = getattr(
        CONF, 'artifact_type:' + af.get_type_name()).default_store
    # use global parameter if default store isn't set per artifact type
    if default_store is None:
        default_store = CONF.glance_store.default_store

    # try to perform blob uploading to storage backend
    try:
        blobs_info = store_api.save_blobs_to_store(
            blobs, context, af.get_max_blob_size(folder_name),
            default_store)
        for name in zip_ref.namelist():
            if not name.endswith('/'):
                location_uri, size, checksums = blobs_info[
                    file_dict[name]['id']]
                # update blob info and activate it
                file_dict[name].update({'url': location_uri,
                                        'status': 'active',
                                        'size': size})
                file_dict[name].update(checksums)
    except Exception:
        # if upload failed remove blob from db and storage
        with excutils.save_and_reraise_exception(logger=LOG):
            af.update_blob(context, af.id, folder_name, None)

    af.update_blob(context, af.id, folder_name, file_dict)


def upload_content_file(context, af, data, blob_dict, key_name,
                        content_type='application/octet-stream'):
    """Upload a file to a blob dictionary.

    :param context: user context
    :param af: artifact object
    :param data: bytes that need to be stored in the blob dictionary
    :param blob_dict: name of the blob_dictionary field
    :param key_name: name of key in the dictionary
    :param content_type: (optional) specifies mime type of uploading data
    """
    blob_id = uuidutils.generate_uuid()
    # create an an empty blob instance in db with 'saving' status
    blob = {'url': None, 'size': None, 'md5': None, 'sha1': None,
            'sha256': None, 'status': glare_fields.BlobFieldType.SAVING,
            'external': False, 'content_type': content_type, 'id': blob_id}

    getattr(af, blob_dict)[key_name] = blob
    af = af.update_blob(context, af.id, blob_dict, getattr(af, blob_dict))

    # try to perform blob uploading to storage backend
    try:
        default_store = getattr(
            CONF, 'artifact_type:' + af.get_type_name()).default_store
        # use global parameter if default store isn't set per artifact type
        if default_store is None:
            default_store = CONF.glance_store.default_store

        location_uri, size, checksums = store_api.save_blob_to_store(
            blob_id, data, context, af.get_max_blob_size(blob_dict),
            default_store)
    except Exception:
        # if upload failed remove blob from db and storage
        with excutils.save_and_reraise_exception(logger=LOG):
            del getattr(af, blob_dict)[key_name]
            af = af.update_blob(context, af.id,
                                blob_dict, getattr(af, blob_dict))
    # update blob info and activate it
    blob.update({'url': location_uri,
                 'status': glare_fields.BlobFieldType.ACTIVE,
                 'size': size})
    blob.update(checksums)
    getattr(af, blob_dict)[key_name] = blob
    af.update_blob(context, af.id, blob_dict, getattr(af, blob_dict))



# Copyright (c) 2015 Mirantis, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from tempest import clients
from tempest.common import credentials_factory as common_creds
from tempest import config


from glare_tempest_plugin.services.artifacts import artifacts_client

CONF = config.CONF

ADMIN_CREDS = None


class Manager(clients.Manager):

    def __init__(self, credentials=None):
        if credentials is None:
            global ADMIN_CREDS
            if ADMIN_CREDS is None:
                ADMIN_CREDS = common_creds.get_configured_admin_credentials()
            credentials = ADMIN_CREDS
        super(Manager, self).__init__(credentials)

        self.artifacts_client = artifacts_client.ArtifactsClient(
            self.auth_provider)



# Copyright 2017 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from oslo_versionedobjects import fields

from glare.objects.meta import fields as glare_fields
from glare.objects.meta import validators
from glare.tests.unit import base


class TestValidators(base.BaseTestArtifactAPI):

    """Class for testing field validators."""

    def test_uuid(self):
        # test if applied string is uuid4
        validator = validators.UUID()

        # valid string - no exception
        validator('167f8083-6bef-4f37-bf04-250343a2d53c')

        # invalid string - ValueError
        self.assertRaises(ValueError, validator, 'INVALID')

        # only strings can be applied as values
        self.assertEqual((fields.StringField,),
                         validators.UUID.get_allowed_types())

        self.assertEqual(
            {'pattern': ('^([0-9a-fA-F]){8}-([0-9a-fA-F]){4}-([0-9a-fA-F])'
                         '{4}-([0-9a-fA-F]){4}-([0-9a-fA-F]){12}$')},
            validator.to_jsonschema())

    def test_regex(self):
        # test regex '^([0-9a-fA-F]){8}$'
        validator = validators.Regex('^([0-9a-fA-F]){8}$')

        # valid string - no exception
        validator('167f8083')

        # invalid string - ValueError
        self.assertRaises(ValueError, validator, 'INVALID')
        self.assertRaises(ValueError, validator, '167f808Z')
        self.assertRaises(ValueError, validator, '167f80835')

        # only strings can be applied as values
        self.assertEqual((fields.StringField,),
                         validators.UUID.get_allowed_types())

        self.assertEqual(
            {'pattern': '^([0-9a-fA-F]){8}$'},
            validator.to_jsonschema())

    def test_allowed_values(self):
        # test that field may have preoccupied values
        validator_s = validators.AllowedValues(['aaa', 'bbb'])
        validator_i = validators.AllowedValues([1, 2, 3])
        validator_f = validators.AllowedValues([1.0, 2.0, 3.0])

        # allowed value - no exception
        validator_s('aaa')
        validator_s('bbb')
        validator_i(1)
        validator_i(3)
        validator_f(1.0)
        validator_f(3.0)

        # not allowed value - value error
        self.assertRaises(ValueError, validator_s, 'a')
        self.assertRaises(ValueError, validator_i, 4)
        self.assertRaises(ValueError, validator_f, 4.0)

        # only strings, integers and floats can be applied as values
        self.assertEqual(
            (fields.StringField, fields.IntegerField, fields.FloatField),
            validators.AllowedValues.get_allowed_types())

        self.assertEqual({'enum': ['aaa', 'bbb']}, validator_s.to_jsonschema())
        self.assertEqual({'enum': [1, 2, 3]}, validator_i.to_jsonschema())
        self.assertEqual({'enum': [1.0, 2.0, 3.0]},
                         validator_f.to_jsonschema())

    def test_max_str_len(self):
        # test max allowed string length
        validator = validators.MaxStrLen(10)

        # allowed length - no exception
        validator('a' * 10)
        validator('')

        # too long string - value error
        self.assertRaises(ValueError, validator, 'a' * 11)

        # only strings can be applied as values
        self.assertEqual((fields.StringField,),
                         validators.MaxStrLen.get_allowed_types())

        self.assertEqual({'maxLength': 10}, validator.to_jsonschema())

    def test_min_str_len(self):
        # test min allowed string length
        validator = validators.MinStrLen(10)

        # allowed length - no exception
        validator('a' * 10)

        # too short string - value error
        self.assertRaises(ValueError, validator, 'a' * 9)
        self.assertRaises(ValueError, validator, '')

        # only strings can be applied as values
        self.assertEqual((fields.StringField,),
                         validators.MinStrLen.get_allowed_types())

        self.assertEqual({'minLength': 10}, validator.to_jsonschema())

    def test_forbidden_chars(self):
        # test that string has no forbidden chars
        validator = validators.ForbiddenChars(['a', '?'])

        # allowed length - no exception
        validator('b' * 10)

        # string contains forbidden chars - value error
        self.assertRaises(ValueError, validator, 'abc')
        self.assertRaises(ValueError, validator, '?')

        # only strings can be applied as values
        self.assertEqual((fields.StringField,),
                         validators.ForbiddenChars.get_allowed_types())

        self.assertEqual({'pattern': '^[^a?]+$'}, validator.to_jsonschema())

    def test_max_dict_size(self):
        # test max dict size
        validator = validators.MaxDictSize(3)

        # allowed size - no exception
        validator({'a': 1, 'b': 2, 'c': 3})
        validator({})

        # too big dictionary - value error
        self.assertRaises(ValueError, validator,
                          {'a': 1, 'b': 2, 'c': 3, 'd': 4})

        # only dicts can be applied as values
        self.assertEqual((glare_fields.Dict,),
                         validators.MaxDictSize.get_allowed_types())

        self.assertEqual({'maxProperties': 3}, validator.to_jsonschema())

    def test_min_dict_size(self):
        # test min dict size
        validator = validators.MinDictSize(3)

        # allowed size - no exception
        validator({'a': 1, 'b': 2, 'c': 3})

        # too small dictionary - value error
        self.assertRaises(ValueError, validator,
                          {'a': 1, 'b': 2})
        self.assertRaises(ValueError, validator, {})

        # only dicts can be applied as values
        self.assertEqual((glare_fields.Dict,),
                         validators.MinDictSize.get_allowed_types())

        self.assertEqual({'minProperties': 3}, validator.to_jsonschema())

    def test_max_list_size(self):
        # test max list size
        validator = validators.MaxListSize(3)

        # allowed size - no exception
        validator(['a', 'b', 'c'])
        validator([])

        # too big list - value error
        self.assertRaises(ValueError, validator,
                          ['a', 'b', 'c', 'd'])

        # only lists can be applied as values
        self.assertEqual((glare_fields.List,),
                         validators.MaxListSize.get_allowed_types())

        self.assertEqual({'maxItems': 3}, validator.to_jsonschema())

    def test_min_list_size(self):
        # test max list size
        validator = validators.MinListSize(3)

        # allowed size - no exception
        validator(['a', 'b', 'c'])

        # too small list - value error
        self.assertRaises(ValueError, validator, ['a', 'b'])
        self.assertRaises(ValueError, validator, [])

        # only lists can be applied as values
        self.assertEqual((glare_fields.List,),
                         validators.MinListSize.get_allowed_types())

        self.assertEqual({'minItems': 3}, validator.to_jsonschema())

    def test_max_number_size(self):
        # test max number size
        validator = validators.MaxNumberSize(10)

        # allowed size - no exception
        validator(10)
        validator(0)
        validator(10.0)
        validator(0.0)

        # too big number - value error
        self.assertRaises(ValueError, validator, 11)
        self.assertRaises(ValueError, validator, 10.1)

        # only integers and floats can be applied as values
        self.assertEqual((fields.IntegerField, fields.FloatField),
                         validators.MaxNumberSize.get_allowed_types())

        self.assertEqual({'maximum': 10}, validator.to_jsonschema())

    def test_min_number_size(self):
        # test min number size
        validator = validators.MinNumberSize(10)

        # allowed size - no exception
        validator(10)
        validator(10.0)

        # too small number - value error
        self.assertRaises(ValueError, validator, 9)
        self.assertRaises(ValueError, validator, 9.9)
        self.assertRaises(ValueError, validator, 0)
        self.assertRaises(ValueError, validator, 0)

        # only integers and floats can be applied as values
        self.assertEqual((fields.IntegerField, fields.FloatField),
                         validators.MinNumberSize.get_allowed_types())

        self.assertEqual({'minimum': 10}, validator.to_jsonschema())

    def test_unique(self):
        # test uniqueness of list elements

        # validator raises exception in case of duplicates in the list
        validator = validators.Unique()
        # non strict validator removes duplicates without raising of ValueError
        validator_nonstrict = validators.Unique(convert_to_set=True)

        # all elements unique - no exception
        validator(['a', 'b', 'c'])
        validator([])

        # duplicates in the list - value error
        self.assertRaises(ValueError, validator, ['a', 'a', 'b'])

        # non-strict validator converts list to set of elements
        l = ['a', 'a', 'b']
        validator_nonstrict(l)
        self.assertEqual({'a', 'b'}, set(l))

        # only lists can be applied as values
        self.assertEqual((glare_fields.List,),
                         validators.Unique.get_allowed_types())

        self.assertEqual({'uniqueItems': True}, validator.to_jsonschema())

    def test_allowed_dict_keys(self):
        # test that dictionary contains only allowed keys
        validator = validators.AllowedDictKeys(['aaa', 'bbb', 'ccc'])

        # only allowed keys - no exception
        validator({'aaa': 5, 'bbb': 6})
        validator({})

        # if dictionary has other keys - value error
        self.assertRaises(ValueError, validator, {'aaa': 5, 'a': 7, 'bbb': 6})

        # only dicts can be applied as values
        self.assertEqual((glare_fields.Dict,),
                         validators.AllowedDictKeys.get_allowed_types())

        self.assertEqual({'properties': {'aaa': {}, 'bbb': {}, 'ccc': {}}},
                         validator.to_jsonschema())

    def test_required_dict_keys(self):
        # test that dictionary has required keys
        validator = validators.RequiredDictKeys(['aaa', 'bbb'])

        # if dict has required keys - no exception
        validator({'aaa': 5, 'bbb': 6})
        validator({'aaa': 5, 'bbb': 6, 'ccc': 7})

        # in other case - value error
        self.assertRaises(ValueError, validator, {'aaa': 5, 'a': 7})
        self.assertRaises(ValueError, validator, {})

        # only dicts can be applied as values
        self.assertEqual((glare_fields.Dict,),
                         validators.RequiredDictKeys.get_allowed_types())

        self.assertEqual({'required': ['aaa', 'bbb']},
                         validator.to_jsonschema())

    def test_max_dict_key_len(self):
        # test max limit for dict key length
        validator = validators.MaxDictKeyLen(5)

        # if key length less than the limit - no exception
        validator({'aaaaa': 5, 'bbbbb': 4})

        # in other case - value error
        self.assertRaises(ValueError, validator, {'aaaaaa': 5, 'a': 7})

        # only dicts can be applied as values
        self.assertEqual((glare_fields.Dict,),
                         validators.MaxDictKeyLen.get_allowed_types())

    def test_mix_dict_key_len(self):
        # test min limit for dict key length
        validator = validators.MinDictKeyLen(5)

        # if key length bigger than the limit - no exception
        validator({'aaaaa': 5, 'bbbbb': 4})

        # in other case - value error
        self.assertRaises(ValueError, validator, {'aaaaa': 5, 'a': 7})

        # only dicts can be applied as values
        self.assertEqual((glare_fields.Dict,),
                         validators.MinDictKeyLen.get_allowed_types())

    def test_allowed_list_values(self):
        # test that list contains only allowed values
        # AllowedValues validator will be applied to each element of the list
        validator = validators.ListElementValidator(
            [validators.AllowedValues(['aaa', 'bbb', 'ccc'])])

        # only allowed values - no exception
        validator(['aaa', 'bbb'])
        validator([])

        # if list has other values - value error
        self.assertRaises(ValueError, validator, ['aaa', 'a', 'bbb'])
        self.assertRaises(ValueError, validator, ['ccc', {'aaa': 'bbb'}])

        # only lists can be applied as values
        self.assertEqual((glare_fields.List,),
                         validators.ListElementValidator.get_allowed_types())

        self.assertEqual({'itemValidators': [{'enum': ['aaa', 'bbb', 'ccc']}]},
                         validator.to_jsonschema())

    def test_allowed_dict_values(self):
        # test that dict contains only allowed values
        # AllowedValues validator will be applied to each element of the dict
        validator = validators.DictElementValidator(
            [validators.AllowedValues(['aaa', 'bbb', 'ccc'])])

        # only allowed values - no exception
        validator({'a': 'aaa', 'b': 'bbb'})
        validator({})

        # if dict has other values - value error
        self.assertRaises(ValueError, validator,
                          {'a': 'aaa', 'b': 'bbb', 'c': 'c'})

        # only dict can be applied as values
        self.assertEqual((glare_fields.Dict,),
                         validators.DictElementValidator.get_allowed_types())

        self.assertEqual(
            {'propertyValidators': [{'enum': ['aaa', 'bbb', 'ccc']}]},
            validator.to_jsonschema())



# Copyright 2017 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import webob

from glare.api.middleware import context
from glare.common import exception as exc
from glare.tests.unit import base


class TestTrustedAuthMiddleware(base.BaseTestCase):
    def _build_request(self, token):
        req = webob.Request.blank("/")
        req.headers["x-auth-token"] = token
        req.get_response = lambda app: None
        return req

    def _build_middleware(self):
        return context.TrustedAuthMiddleware(None)

    def test_header_parsing(self):
        token = 'user1:tenant1:role1,role2'
        req = self._build_request(token)
        self._build_middleware().process_request(req)

        self.assertEqual("Confirmed", req.headers["X-Identity-Status"])
        self.assertEqual("user1", req.headers["X-User-Id"])
        self.assertEqual("tenant1", req.headers["X-Tenant-Id"])
        self.assertEqual("role1,role2", req.headers["X-Roles"])

        self.assertEqual(token, req.context.auth_token)
        self.assertEqual('user1', req.context.user)
        self.assertEqual('tenant1', req.context.tenant)
        self.assertEqual(['role1', 'role2'], req.context.roles)
        self.assertIn('service_catalog', req.context.to_dict())

    def test_no_auth_token(self):
        req = self._build_request(None)
        del req.headers['x-auth-token']
        self.assertRaises(exc.Unauthorized,
                          self._build_middleware().process_request, req)

    def test_wrong_format(self):
        req = self._build_request('WRONG_FORMAT')
        middleware = self._build_middleware()
        self.assertRaises(exc.Unauthorized,
                          middleware.process_request, req)

        req = self._build_request('user1:tenant1:role1:role2')
        self.assertRaises(exc.Unauthorized,
                          middleware.process_request, req)

    def test_no_tenant(self):
        req = self._build_request('user1::role')
        middleware = self._build_middleware()
        self.assertRaises(exc.Unauthorized,
                          middleware.process_request, req)

    def test_no_roles(self):
        # stripping extra spaces in request
        req = self._build_request('user1:tenant1:')
        self._build_middleware().process_request(req)
        self.assertFalse(req.context.is_admin)
        self.assertEqual('user1', req.context.user)
        self.assertEqual("user1", req.headers["X-User-Id"])
        self.assertEqual("", req.headers["X-Roles"])
        self.assertEqual([], req.context.roles)

    def test_is_admin_flag(self):
        # is_admin check should look for 'admin' role by default
        req = self._build_request('user1:tenant1:role1,admin')
        self._build_middleware().process_request(req)
        self.assertTrue(req.context.is_admin)

        # without the 'admin' role, is_admin should be False
        req = self._build_request('user1:tenant1:role1,role2')
        self._build_middleware().process_request(req)
        self.assertFalse(req.context.is_admin)

        # if we change the admin_role attribute, we should be able to use it
        req = self._build_request('user1:tenant1:role1,role2')
        self.policy({'context_is_admin': 'role:role1'})
        self._build_middleware().process_request(req)
        self.assertTrue(req.context.is_admin)

    def test_roles_case_insensitive(self):
        # accept role from request
        req = self._build_request('user1:tenant1:Admin,role2')
        self._build_middleware().process_request(req)
        self.assertTrue(req.context.is_admin)

        # accept role from config
        req = self._build_request('user1:tenant1:role1,role2')
        self.policy({'context_is_admin': 'role:rOLe1'})
        self._build_middleware().process_request(req)
        self.assertTrue(req.context.is_admin)

    def test_token_stripping(self):
        # stripping extra spaces in request
        req = self._build_request('   user1:tenant1:role1\t')
        self.policy({'context_is_admin': 'role:role1'})
        self._build_middleware().process_request(req)
        self.assertTrue(req.context.is_admin)
        self.assertEqual('user1', req.context.user)
        self.assertEqual("user1", req.headers["X-User-Id"])
        self.assertEqual("role1", req.headers["X-Roles"])

    def test_anonymous_access_enabled(self):
        req = self._build_request('user1:none:role1,role2')
        self.config(allow_anonymous_access=True)
        middleware = self._build_middleware()
        middleware.process_request(req)
        self.assertIsNone(req.context.auth_token)
        self.assertIsNone(req.context.user)
        self.assertIsNone(req.context.tenant)
        self.assertEqual([], req.context.roles)
        self.assertFalse(req.context.is_admin)
        self.assertTrue(req.context.read_only)

    def test_anonymous_access_defaults_to_disabled(self):
        req = self._build_request('user1:none:role1,role2')
        middleware = self._build_middleware()
        self.assertRaises(exc.Unauthorized,
                          middleware.process_request, req)

    def test_response(self):
        req = self._build_request('user1:tenant1:role1,role2')
        req.context = context.RequestContext()
        request_id = req.context.request_id

        resp = webob.Response()
        resp.request = req
        self._build_middleware().process_response(resp)
        self.assertEqual(request_id, resp.headers['x-openstack-request-id'])
        resp_req_id = resp.headers['x-openstack-request-id']
        # Validate that request-id do not starts with 'req-req-'
        if isinstance(resp_req_id, bytes):
            resp_req_id = resp_req_id.decode('utf-8')
        self.assertFalse(resp_req_id.startswith('req-req-'))
        self.assertTrue(resp_req_id.startswith('req-'))

    def test_response_no_request_id(self):
        req = self._build_request('user1:tenant1:role1,role2')
        req.context = context.RequestContext()
        del req.context.request_id

        resp = webob.Response()
        resp.request = req
        self._build_middleware().process_response(resp)
        self.assertNotIn('x-openstack-request-id', resp.headers)

    def test_response_no_request_id_prefix(self):
        # prefix is 'req-'
        req = self._build_request('user1:tenant1:role1,role2')
        req.context = context.RequestContext()
        req.context.request_id = "STRING_WITH_NO_PREFIX"

        resp = webob.Response()
        resp.request = req
        self._build_middleware().process_response(resp)
        self.assertEqual('req-STRING_WITH_NO_PREFIX',
                         resp.headers['x-openstack-request-id'])



# Copyright 2017 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from six import BytesIO

from glare.common import exception as exc
from glare.tests.unit import base


class TestDeleteBlobs(base.BaseTestArtifactAPI):
    """Test deleting of custom locations."""

    def setUp(self):
        super(TestDeleteBlobs, self).setUp()
        values = {'name': 'ttt', 'version': '1.0', 'int1': '10'}
        self.sample_artifact = self.controller.create(
            self.req, 'sample_artifact', values)
        self.ct = 'application/vnd+openstack.glare-custom-location+json'

    def test_delete_external_blob(self):
        # Add external location
        body = {'url': 'https://FAKE_LOCATION.com',
                'md5': "fake", 'sha1': "fake_sha", "sha256": "fake_sha256"}
        self.controller.upload_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'blob', body, self.ct)
        art = self.controller.show(self.req, 'sample_artifact',
                                   self.sample_artifact['id'])
        self.assertEqual('active', art['blob']['status'])
        self.assertEqual('fake', art['blob']['md5'])
        self.assertEqual('fake_sha', art['blob']['sha1'])
        self.assertEqual('fake_sha256', art['blob']['sha256'])
        self.assertIsNone(art['blob']['size'])
        self.assertIsNone(art['blob']['content_type'])
        self.assertEqual('https://FAKE_LOCATION.com',
                         art['blob']['url'])
        self.assertNotIn('id', art['blob'])

        # Delete external blob works
        self.controller.delete_external_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'], 'blob')

        art = self.controller.show(self.req, 'sample_artifact',
                                   self.sample_artifact['id'])
        self.assertIsNone(art['blob'])

    def test_delete_external_blob_dict(self):
        # Add external location to the folder
        body = {'url': 'https://FAKE_LOCATION.com',
                'md5': "fake", 'sha1': "fake_sha", "sha256": "fake_sha256"}
        self.controller.upload_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'dict_of_blobs/blob', body, self.ct)
        art = self.controller.show(self.req, 'sample_artifact',
                                   self.sample_artifact['id'])
        self.assertEqual('active', art['dict_of_blobs']['blob']['status'])
        self.assertEqual('fake', art['dict_of_blobs']['blob']['md5'])
        self.assertEqual('fake_sha', art['dict_of_blobs']['blob']['sha1'])
        self.assertEqual('fake_sha256',
                         art['dict_of_blobs']['blob']['sha256'])
        self.assertIsNone(art['dict_of_blobs']['blob']['size'])
        self.assertIsNone(art['dict_of_blobs']['blob']['content_type'])
        self.assertEqual('https://FAKE_LOCATION.com',
                         art['dict_of_blobs']['blob']['url'])
        self.assertNotIn('id', art['blob'])

        # Delete external blob works
        self.controller.delete_external_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'dict_of_blobs/blob')

        art = self.controller.show(self.req, 'sample_artifact',
                                   self.sample_artifact['id'])
        self.assertNotIn('blob', art['dict_of_blobs'])

    def test_delete_internal_blob(self):
        # Upload data to regular blob
        self.controller.upload_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'], 'blob',
            BytesIO(b'aaa'), 'application/octet-stream')
        artifact = self.controller.show(self.req, 'sample_artifact',
                                        self.sample_artifact['id'])
        self.assertEqual(3, artifact['blob']['size'])
        self.assertEqual('active', artifact['blob']['status'])

        # Deletion of uploaded internal blobs fails with Forbidden
        self.assertRaises(
            exc.Forbidden, self.controller.delete_external_blob,
            self.req, 'sample_artifact', self.sample_artifact['id'], 'blob')

    def test_delete_internal_blob_dict(self):
        # Upload data to the blob dict
        self.controller.upload_blob(
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'dict_of_blobs/blob', BytesIO(b'aaa'), 'application/octet-stream')
        artifact = self.controller.show(self.req, 'sample_artifact',
                                        self.sample_artifact['id'])
        self.assertEqual(3, artifact['dict_of_blobs']['blob']['size'])
        self.assertEqual('active', artifact['dict_of_blobs']['blob']['status'])

        # Deletion of uploaded internal blobs fails with Forbidden
        self.assertRaises(
            exc.Forbidden, self.controller.delete_external_blob,
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'dict_of_blobs/blob')

    def test_delete_blob_wrong(self):
        # Non-blob field
        self.assertRaises(
            exc.BadRequest, self.controller.delete_external_blob,
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'int1')

        # Non-existing field
        self.assertRaises(
            exc.BadRequest, self.controller.delete_external_blob,
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'Nonexisting')

        # Empty blob
        self.assertRaises(
            exc.NotFound, self.controller.delete_external_blob,
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'blob')

        # No blob in the blob dict
        self.assertRaises(
            exc.NotFound, self.controller.delete_external_blob,
            self.req, 'sample_artifact', self.sample_artifact['id'],
            'dict_of_blobs/Nonexisting')



# Copyright 2017 - Nokia Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import mock
import requests
import webob

from glare.api.middleware import keycloak_auth
from glare.common import exception as exc
from glare.tests.unit import base


class TestKeycloakAuthMiddleware(base.BaseTestCase):
    def _build_request(self, token):
        req = webob.Request.blank("/")
        req.headers["x-auth-token"] = token
        req.get_response = lambda app: None
        return req

    def _build_middleware(self):
        return keycloak_auth.KeycloakAuthMiddleware(None)

    @mock.patch("requests.get")
    def test_header_parsing(self, mocked_get):
        token = {
            "iss": "http://localhost:8080/auth/realms/my_realm",
            "realm_access": {
                "roles": ["role1", "role2"]
            }
        }
        mocked_resp = mock.Mock()
        mocked_resp.status_code = 200
        mocked_resp.json.return_value = '{"user": "mike"}'
        mocked_get.return_value = mocked_resp

        req = self._build_request(token)
        with mock.patch("jwt.decode", return_value=token):
            self._build_middleware()(req)
        self.assertEqual("Confirmed", req.headers["X-Identity-Status"])
        self.assertEqual("my_realm", req.headers["X-Project-Id"])
        self.assertEqual("role1,role2", req.headers["X-Roles"])
        self.assertEqual(1, mocked_get.call_count)

    def test_no_auth_token(self):
        req = webob.Request.blank("/")
        self.assertRaises(exc.Unauthorized, self._build_middleware(), req)

    @mock.patch("requests.get")
    def test_no_realm_access(self, mocked_get):
        token = {
            "iss": "http://localhost:8080/auth/realms/my_realm",
        }
        mocked_resp = mock.Mock()
        mocked_resp.status_code = 200
        mocked_resp.json.return_value = '{"user": "mike"}'
        mocked_get.return_value = mocked_resp

        req = self._build_request(token)
        with mock.patch("jwt.decode", return_value=token):
            self._build_middleware()(req)
        self.assertEqual("Confirmed", req.headers["X-Identity-Status"])
        self.assertEqual("my_realm", req.headers["X-Project-Id"])
        self.assertEqual("", req.headers["X-Roles"])

    def test_wrong_token_format(self):
        req = self._build_request(token="WRONG_FORMAT_TOKEN")
        self.assertRaises(exc.Unauthorized, self._build_middleware(), req)

    @mock.patch("requests.get")
    def test_server_unauthorized(self, mocked_get):
        token = {
            "iss": "http://localhost:8080/auth/realms/my_realm",
        }
        mocked_resp = mock.Mock()
        mocked_resp.status_code = 401
        mocked_resp.json.return_value = '{"user": "mike"}'
        mocked_get.return_value = mocked_resp

        req = self._build_request(token)
        with mock.patch("jwt.decode", return_value=token):
            self.assertRaises(exc.Unauthorized, self._build_middleware(), req)

    @mock.patch("requests.get")
    def test_server_forbidden(self, mocked_get):
        token = {
            "iss": "http://localhost:8080/auth/realms/my_realm",
        }
        mocked_resp = mock.Mock()
        mocked_resp.status_code = 403
        mocked_resp.json.return_value = '{"user": "mike"}'
        mocked_get.return_value = mocked_resp

        req = self._build_request(token)
        with mock.patch("jwt.decode", return_value=token):
            self.assertRaises(exc.Forbidden, self._build_middleware(), req)

    @mock.patch("requests.get")
    def test_server_exception(self, mocked_get):
        token = {
            "iss": "http://localhost:8080/auth/realms/my_realm",
        }
        mocked_resp = mock.Mock()
        mocked_resp.status_code = 500
        mocked_resp.json.return_value = '{"user": "mike"}'
        mocked_get.return_value = mocked_resp

        req = self._build_request(token)
        with mock.patch("jwt.decode", return_value=token):
            self.assertRaises(
                exc.GlareException, self._build_middleware(), req)

    @mock.patch("requests.get")
    def test_connection_error(self, mocked_get):
        token = {
            "iss": "http://localhost:8080/auth/realms/my_realm",
        }
        mocked_get.side_effect = requests.ConnectionError

        req = self._build_request(token)
        with mock.patch("jwt.decode", return_value=token):
            self.assertRaises(
                exc.GlareException, self._build_middleware(), req)

    @mock.patch("requests.get")
    def test_userinfo_endpoint_empty(self, mocked_get):
        self.config(user_info_endpoint_url='',
                    group='keycloak_oidc')
        token = {
            "iss": "http://localhost:8080/auth/realms/my_realm",
            "realm_access": {
                "roles": ["role1", "role2"]
            }
        }

        req = self._build_request(token)
        with mock.patch("jwt.decode", return_value=token):
            self._build_middleware()(req)
        self.assertEqual("Confirmed", req.headers["X-Identity-Status"])
        self.assertEqual("my_realm", req.headers["X-Project-Id"])
        self.assertEqual("role1,role2", req.headers["X-Roles"])
        self.assertEqual(0, mocked_get.call_count)



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""This file contains classes that wrap nat"""

from oslo_versionedobjects import fields

from glare.common import exception as exc
from glare.objects.meta import fields as glare_fields
from glare.objects.meta import validators as val_lib

FILTERS = (
    FILTER_EQ, FILTER_NEQ, FILTER_IN, FILTER_GT, FILTER_GTE, FILTER_LT,
    FILTER_LTE) = ('eq', 'neq', 'in', 'gt', 'gte', 'lt', 'lte')

DEFAULT_MAX_BLOB_SIZE = 10485760  # 10 Megabytes
DEFAULT_MAX_FOLDER_SIZE = 2673868800  # 2550 Megabytes


class Field(object):
    def __init__(self, field_class, mutable=False, required_on_activate=True,
                 system=False, validators=None, nullable=True, default=None,
                 sortable=False, filter_ops=None, description=""):
        """Init and validate field.
        Each artifact field has several common properties:

        :param required_on_activate: boolean value indicating if the field
         value should be specified for the artifact
         before activation (Default:True).
        :param mutable: boolean value indicating if the field value may
         be changed after the artifact is activated. (Default: False)
        :param system: boolean value indicating if the field
         value cannot be edited by user (Default: False).
        :param sortable: boolean value indicating if there is a possibility
         to sort by this fields's values. (Default: False) Only fields of
         4 primitive types may be sortable: integer, string, float and boolean.
        :param default: a default value for the field may be specified
         (Default: None).
        :param validators: a list of objects. When user sets a value to
         the field with additional validators Glare checks them before setting
         the value and raises ValueError if at least one of the requirements
         is not satisfied.
        :param filter_ops: a list of available filter operators for the field.
         There are seven available operators:
         'eq', 'neq', 'lt', 'lte', 'gt', 'gte', 'in'.
         """

        if not issubclass(field_class, fields.AutoTypedField):
            raise exc.IncorrectArtifactType(
                "Field class %s must be sub-class of AutoTypedField." %
                field_class)

        self.validators = validators or []
        for v in self.validators:
            v.check_type_allowed(field_class)
            if isinstance(v, val_lib.MaxStrLen):
                if v.size > 255 and sortable:
                    raise exc.IncorrectArtifactType(
                        "It's forbidden to make field %(field)s "
                        "sortable if string length can be more than 255 "
                        "symbols. Maximal allowed length now: %(max)d" %
                        {"field": str(field_class), 'max': v.size})

        self.field_class = field_class
        self.nullable = nullable
        self.default = default
        self.vo_props = ['nullable', 'default']

        self.mutable = mutable
        self.required_on_activate = required_on_activate
        self.system = system
        self.sortable = sortable

        try:
            default_ops = self.get_allowed_filter_ops(self.element_type)
        except AttributeError:
            default_ops = self.get_allowed_filter_ops(field_class)

        if filter_ops is None:
            self.filter_ops = default_ops
        else:
            for op in filter_ops:
                if op not in default_ops:
                    raise exc.IncorrectArtifactType(
                        "Incorrect filter operator '%s'. "
                        "Only %s are allowed" % (op, ', '.join(default_ops)))
            self.filter_ops = filter_ops

        self.field_props = ['mutable', 'required_on_activate', 'system',
                            'sortable', 'filter_ops', 'description']
        self.description = description

    @staticmethod
    def get_allowed_filter_ops(field):
        if field in (fields.StringField, fields.String):
            return [FILTER_EQ, FILTER_NEQ, FILTER_IN]
        elif field in (fields.IntegerField, fields.Integer, fields.FloatField,
                       fields.Float, glare_fields.VersionField):
            return FILTERS
        elif field in (fields.FlexibleBooleanField, fields.FlexibleBoolean,
                       glare_fields.Link, glare_fields.LinkFieldType):
            return [FILTER_EQ, FILTER_NEQ]
        elif field in (glare_fields.BlobField, glare_fields.BlobFieldType):
            return []
        elif field is fields.DateTimeField:
            return [FILTER_LT, FILTER_GT]

    def get_default_validators(self):
        default = []
        if issubclass(self.field_class, fields.StringField):
            # check if fields is string
            if not any(isinstance(v, val_lib.MaxStrLen)
                       for v in self.validators) and \
                    not any(isinstance(v, val_lib.AllowedValues)
                            for v in self.validators):
                default.append(val_lib.MaxStrLen(255))
        return default

    def get_field(self):
        # init the field
        vo_props = {prop_name: getattr(self, prop_name)
                    for prop_name in self.vo_props}
        field = self.field_class(**vo_props)
        # setup custom field properties
        field_props = {prop_name: getattr(self, prop_name)
                       for prop_name in self.field_props}
        for prop, value in field_props.items():
                setattr(field, prop, value)

        # apply custom validators
        vals = self.validators
        for def_val in self.get_default_validators():
            for val in self.validators:
                if type(val) is type(def_val):
                    break
            else:
                vals.append(def_val)

        def wrapper(coerce_func):
            def coerce_wrapper(obj, field, value):
                try:
                    val = coerce_func(obj, field, value)
                    if val is not None:
                        for check_func in vals:
                            check_func(val)
                    return val
                except (KeyError, ValueError, TypeError) as e:
                    msg = "Type: %s. Field: %s. Exception: %s" % (
                        obj.get_type_name(), field, str(e))
                    raise exc.BadRequest(message=msg)
            return coerce_wrapper

        field.coerce = wrapper(field.coerce)
        field.validators = vals
        return field

    @classmethod
    def init(cls, *args, **kwargs):
        """Fabric to build fields."""
        return cls(*args, **kwargs).get_field()


class CompoundField(Field):
    def __init__(self, field_class, element_type, element_validators=None,
                 **kwargs):
        if element_type is None:
            raise exc.IncorrectArtifactType("'element_type' must be set for "
                                            "compound type.")
        self.element_type = element_type

        super(CompoundField, self).__init__(field_class, **kwargs)

        self.vo_props.append('element_type')
        self.field_props.append('element_type')

        self.element_validators = element_validators or []
        if self.sortable:
            raise exc.IncorrectArtifactType("'sortable' must be False for "
                                            "compound type.")

    def get_element_validators(self):
        default_vals = []
        if issubclass(self.element_type, fields.String):
            # check if fields is string
            if not any(isinstance(v, val_lib.MaxStrLen)
                       for v in self.element_validators):
                default_vals.append(val_lib.MaxStrLen(255))
        vals = default_vals + self.element_validators
        for v in vals:
            v.check_type_allowed(self.element_type)
        return default_vals + self.element_validators


class ListField(CompoundField):
    def __init__(self, element_type, max_size=255, **kwargs):
        if 'default' not in kwargs:
            kwargs['default'] = []
        if element_type is glare_fields.BlobField:
            raise exc.IncorrectArtifactType("List of blobs is not allowed "
                                            "to be specified in artifact.")
        super(ListField, self).__init__(glare_fields.List, element_type,
                                        **kwargs)
        self.validators.append(val_lib.MaxListSize(max_size))

    def get_default_validators(self):
        default_vals = []
        elem_val = val_lib.ListElementValidator(
            super(ListField, self).get_element_validators())
        default_vals.append(elem_val)
        return default_vals


class DictField(CompoundField):
    def __init__(self, element_type, max_size=255, **kwargs):
        if 'default' not in kwargs:
            kwargs['default'] = {}
        super(DictField, self).__init__(glare_fields.Dict, element_type,
                                        **kwargs)
        self.validators.append(val_lib.MaxDictSize(max_size))

    def get_default_validators(self):
        default_vals = []
        elem_val = val_lib.DictElementValidator(
            super(DictField, self).get_element_validators())
        default_vals.append(elem_val)
        default_vals.append(val_lib.MaxDictKeyLen(255))
        default_vals.append(val_lib.MinDictKeyLen(1))
        return default_vals


class BlobField(Field):
    def __init__(self, max_blob_size=DEFAULT_MAX_BLOB_SIZE, **kwargs):
        super(BlobField, self).__init__(
            field_class=glare_fields.BlobField, **kwargs)
        self.max_blob_size = int(max_blob_size)
        self.field_props.append('max_blob_size')


class FolderField(DictField):
    def __init__(self, max_blob_size=DEFAULT_MAX_BLOB_SIZE,
                 max_folder_size=DEFAULT_MAX_FOLDER_SIZE, **kwargs):
        super(FolderField, self).__init__(
            element_type=glare_fields.BlobFieldType, **kwargs)
        self.max_blob_size = int(max_blob_size)
        self.max_folder_size = int(max_folder_size)
        self.field_props.append('max_blob_size')
        self.field_props.append('max_folder_size')

# Classes below added for backward compatibility. They shouldn't be used

Attribute = Field
CompoundAttribute = CompoundField
ListAttribute = ListField
DictAttribute = DictField
BlobAttribute = BlobField
BlobDictAttribute = FolderField



# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools


from glare_tempest_plugin.tests.api import base
from tempest import config

CONF = config.CONF


class TestListSanity(base.BaseArtifactTest):

    @testtools.testcase.attr('smoke')
    def test_list_artifacts(self):
        art = self.artifacts_client.create_artifact('images', 'tempest_test')
        self.artifacts_client.list_artifacts('images')
        self.artifacts_client.get_artifact('images', art['id'])
        self.artifacts_client.update_artifact(type_name='images',
                                              art_id=art['id'],
                                              name='newnewname')
        data = 'dataaaa'
        self.artifacts_client.upload_blob('images', art['id'], 'image', data)
        self.artifacts_client.download_blob('images', art['id'], 'image')
        self.artifacts_client.delete_artifact('images', art['id'])



# Copyright 2017 Nokia, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import hashlib
import testtools


from glare_tempest_plugin.tests.api import base
from pprint import pformat


class TestDownloadSanity(base.BaseArtifactTest):

    @testtools.testcase.attr('TestDownloadSanity')
    def test_blob_dict(self):
        """Uploading data to a folder and then download it back"""

        # Create a test artifact
        art = self.artifacts_client.create_artifact('sample_artifact',
                                                    'sample_art1')
        data = "data" * 100
        art = self.artifacts_client.upload_blob(
            'sample_artifact', art['id'], '/dict_of_blobs/new_blob', data)

        art_blob = art['dict_of_blobs']['new_blob']
        self.assertEqual(400, art_blob['size'])
        self.assertEqual('active', art_blob['status'], pformat(art_blob))

        encoded_data = data.encode('UTF-8')
        md5 = hashlib.md5(encoded_data).hexdigest()
        sha1 = hashlib.sha1(encoded_data).hexdigest()
        sha256 = hashlib.sha256(encoded_data).hexdigest()
        self.assertEqual(md5, art_blob['md5'])
        self.assertEqual(sha1, art_blob['sha1'])
        self.assertEqual(sha256, art_blob['sha256'])

        # Download data from the folder (dict_of_blobs)
        self.assertEqual(data,
                         self.artifacts_client.download_blob(
                             'sample_artifact', art['id'],
                             '/dict_of_blobs/new_blob'), pformat(art))

    @testtools.testcase.attr('TestDownloadSanity')
    def test_blob_download(self):
        data = 'some_arbitrary_testing_data'
        art = self.artifacts_client.create_artifact('sample_artifact',
                                                    'test_af')

        # upload data
        art = self.artifacts_client.upload_blob('sample_artifact',
                                                art['id'], 'blob', data)

        art_blob = art['blob']
        self.assertEqual('active', art_blob['status'], pformat(art))
        encoded_data = data.encode('UTF-8')
        md5 = hashlib.md5(encoded_data).hexdigest()
        sha1 = hashlib.sha1(encoded_data).hexdigest()
        sha256 = hashlib.sha256(encoded_data).hexdigest()
        self.assertEqual(md5, art_blob['md5'])
        self.assertEqual(sha1, art_blob['sha1'])
        self.assertEqual(sha256, art_blob['sha256'])

        #  Download data
        self.assertEqual(data,
                         self.artifacts_client.download_blob(
                             'sample_artifact', art['id'],
                             '/blob'), pformat(art))



# Copyright (c) 2016 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from oslo_serialization import jsonutils
from oslo_utils import uuidutils
import requests

from glare.tests import functional


def sort_results(lst, target='name'):
    return sorted(lst, key=lambda x: x[target])


class TestArtifact(functional.FunctionalTest):
    enabled_types = (u'sample_artifact', u'images', u'heat_templates',
                     u'heat_environments', u'tosca_templates',
                     u'murano_packages', u'all')

    users = {
        'user1': {
            'id': uuidutils.generate_uuid(),
            'tenant_id': uuidutils.generate_uuid(),
            'token': uuidutils.generate_uuid(),
            'role': 'member'
        },
        'user2': {
            'id': uuidutils.generate_uuid(),
            'tenant_id': uuidutils.generate_uuid(),
            'token': uuidutils.generate_uuid(),
            'role': 'member'
        },
        'admin': {
            'id': uuidutils.generate_uuid(),
            'tenant_id': uuidutils.generate_uuid(),
            'token': uuidutils.generate_uuid(),
            'role': 'admin'
        },
        'anonymous': {
            'id': None,
            'tenant_id': None,
            'token': None,
            'role': None
        }
    }

    def setUp(self):
        super(TestArtifact, self).setUp()

        self.set_user('user1')
        self.glare_server.deployment_flavor = 'noauth'

        self.glare_server.enabled_artifact_types = ','.join(
            self.enabled_types)
        self.glare_server.custom_artifact_types_modules = (
            'glare.tests.sample_artifact')
        self.start_servers(**self.__dict__.copy())

    def tearDown(self):
        self.stop_servers()
        self._reset_database(self.glare_server.sql_connection)
        super(TestArtifact, self).tearDown()

    def _url(self, path):
        if path.startswith('/schemas') or \
                path.startswith('/quotas') or \
                path.startswith('/project-quotas'):
            return 'http://127.0.0.1:%d%s' % (self.glare_port, path)
        else:
            return 'http://127.0.0.1:%d/artifacts%s' % (self.glare_port, path)

    def set_user(self, username):
        if username not in self.users:
            raise KeyError
        self.current_user = username

    def _headers(self, custom_headers=None):
        base_headers = {
            'X-Identity-Status': 'Confirmed',
            'X-Auth-Token': self.users[self.current_user]['token'],
            'X-User-Id': self.users[self.current_user]['id'],
            'X-Tenant-Id': self.users[self.current_user]['tenant_id'],
            'X-Project-Id': self.users[self.current_user]['tenant_id'],
            'X-Roles': self.users[self.current_user]['role'],
        }
        base_headers.update(custom_headers or {})
        return base_headers

    def create_artifact(self, data=None, status=201,
                        type_name='sample_artifact'):
        return self.post('/' + type_name, data or {}, status=status)

    def _check_artifact_method(self, method, url, data=None, status=200,
                               headers=None):
        if not headers:
            headers = self._headers()
        else:
            headers = self._headers(headers)
        headers.setdefault("Content-Type", "application/json")
        if 'application/json' in headers['Content-Type'] and data is not None:
            data = jsonutils.dumps(data)
        response = getattr(requests, method)(self._url(url), headers=headers,
                                             data=data)
        self.assertEqual(status, response.status_code, response.text)
        if status >= 400:
            return response.text
        if ("application/json" in response.headers["content-type"] or
                "application/schema+json" in response.headers["content-type"]):
            return jsonutils.loads(response.text)
        return response.text

    def post(self, url, data=None, status=201, headers=None):
        return self._check_artifact_method("post", url, data, status=status,
                                           headers=headers)

    def get(self, url, status=200, headers=None):
        return self._check_artifact_method("get", url, status=status,
                                           headers=headers)

    def delete(self, url, status=204, headers=None):
        return self._check_artifact_method("delete", url, status=status,
                                           headers=headers)

    def patch(self, url, data, status=200, headers=None):
        if headers is None:
            headers = {}
        if 'Content-Type' not in headers:
            headers.update({'Content-Type': 'application/json-patch+json'})
        return self._check_artifact_method("patch", url, data, status=status,
                                           headers=headers)

    def put(self, url, data=None, status=200, headers=None):
        return self._check_artifact_method("put", url, data, status=status,
                                           headers=headers)

    # the test cases below are written in accordance with use cases
    # each test tries to cover separate use case in Glare
    # all code inside each test tries to cover all operators and data
    # involved in use case execution
    # each tests represents part of artifact lifecycle
    # so we can easily define where is the failed code

    make_active = [{"op": "replace", "path": "/status", "value": "active"}]
    make_deactivated = [{"op": "replace", "path": "/status",
                         "value": "deactivated"}]
    make_public = [{"op": "replace", "path": "/visibility", "value": "public"}]

    def admin_action(self, artifact_id, body, status=200,
                     type_name='sample_artifact'):
        cur_user = self.current_user
        self.set_user('admin')
        url = '/%s/%s' % (type_name, artifact_id)
        af = self.patch(url=url, data=body, status=status)
        self.set_user(cur_user)
        return af



# Copyright 2017 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import hashlib
import requests

from glare.tests.functional import base


class TestMultiStore(base.TestArtifact):

    def setUp(self):
        base.functional.FunctionalTest.setUp(self)

        self.set_user('user1')
        self.glare_server.deployment_flavor = 'noauth'

        self.glare_server.enabled_artifact_types = 'sample_artifact'
        self.glare_server.custom_artifact_types_modules = (
            'glare.tests.sample_artifact')
        self.glare_server.artifact_type_section = """
[artifact_type:sample_artifact]
delayed_delete = False
default_store = database
"""
        self.start_servers(**self.__dict__.copy())

    def test_blob_dicts(self):
        # Getting empty artifact list
        url = '/sample_artifact'
        response = self.get(url=url, status=200)
        expected = {'first': '/artifacts/sample_artifact',
                    'sample_artifact': [],
                    'schema': '/schemas/sample_artifact'}
        self.assertEqual(expected, response)

        # Create a test artifact
        art = self.create_artifact(status=201,
                                   data={'name': 'test',
                                         'version': '1.0',
                                         'string_required': '123'})
        self.assertIsNotNone(art['id'])

        # Get the artifact which should have a generated id and status
        # 'drafted'
        url = '/sample_artifact/%s' % art['id']
        art_1 = self.get(url=url, status=200)
        self.assertIsNotNone(art_1['id'])
        self.assertEqual('drafted', art_1['status'])

        # Upload data to blob dict
        headers = {'Content-Type': 'application/octet-stream'}
        data = "data" * 100

        self.put(url=url + '/dict_of_blobs/new_blob',
                 data=data, status=200, headers=headers)

        # Download data from blob dict
        self.assertEqual(data,
                         self.get(url=url + '/dict_of_blobs/new_blob',
                                  status=200))

        # download blob from undefined dict property
        self.get(url=url + '/not_a_dict/not_a_blob', status=400)

    def test_blob_upload(self):
        # create artifact with blob
        data = 'data'
        self.create_artifact(
            data={'name': 'test_af', 'blob': data,
                  'version': '0.0.1'}, status=400)
        art = self.create_artifact(data={'name': 'test_af',
                                         'version': '0.0.1',
                                         'string_required': 'test'})
        url = '/sample_artifact/%s' % art['id']
        headers = {'Content-Type': 'application/octet-stream'}

        # upload to non-existing property
        self.put(url=url + '/blob_non_exist', data=data, status=400,
                 headers=headers)

        # upload too big value
        big_data = "this is the smallest big data"
        self.put(url=url + '/small_blob', data=big_data, status=413,
                 headers=headers)
        # upload correct blob value
        self.put(url=url + '/small_blob', data=big_data[:2], headers=headers)

        # Upload artifact via different user
        self.set_user('user2')
        self.put(url=url + '/blob', data=data, status=404,
                 headers=headers)

        # Upload file to the artifact
        self.set_user('user1')
        art = self.put(url=url + '/blob', data=data, status=200,
                       headers=headers)
        self.assertEqual('active', art['blob']['status'])
        self.assertEqual('application/octet-stream',
                         art['blob']['content_type'])
        self.assertIn('url', art['blob'])
        self.assertNotIn('id', art['blob'])

        # reUpload file to artifact
        self.put(url=url + '/blob', data=data, status=409,
                 headers=headers)
        # upload blob dict
        self.put(url + '/dict_of_blobs/test_key', data=data, headers=headers)
        # test re-upload failed
        self.put(url + '/dict_of_blobs/test_key', data=data, headers=headers,
                 status=409)

        # upload few other blobs to the dict
        for elem in ('aaa', 'bbb', 'ccc', 'ddd'):
            self.put(url + '/dict_of_blobs/' + elem, data=data,
                     headers=headers)

        # upload to active artifact
        self.patch(url, self.make_active)
        self.put(url + '/dict_of_blobs/key2', data=data, status=403,
                 headers=headers)

        self.delete(url)

    def test_blob_download(self):
        data = 'some_arbitrary_testing_data'
        art = self.create_artifact(data={'name': 'test_af',
                                         'version': '0.0.1'})
        url = '/sample_artifact/%s' % art['id']

        # download not uploaded blob
        self.get(url=url + '/blob', status=404)

        # download blob from not existing artifact
        self.get(url=url + '1/blob', status=404)

        # download blob from undefined property
        self.get(url=url + '/not_a_blob', status=400)

        headers = {'Content-Type': 'application/octet-stream'}
        art = self.put(url=url + '/blob', data=data, status=200,
                       headers=headers)
        self.assertEqual('active', art['blob']['status'])
        md5 = hashlib.md5(data.encode('UTF-8')).hexdigest()
        sha1 = hashlib.sha1(data.encode('UTF-8')).hexdigest()
        sha256 = hashlib.sha256(data.encode('UTF-8')).hexdigest()
        self.assertEqual(md5, art['blob']['md5'])
        self.assertEqual(sha1, art['blob']['sha1'])
        self.assertEqual(sha256, art['blob']['sha256'])

        # check that content-length is in response
        response = requests.get(self._url(url + '/blob'),
                                headers=self._headers())
        self.assertEqual('27', response.headers["content-length"])

        # check that all checksums are in response
        self.assertEqual('0825587cc011b7e76381b65e19d5ec27',
                         response.headers["Content-MD5"])
        self.assertEqual('89eb4b969b721ba8c3aff18ad7d69454f651a697',
                         response.headers["X-Openstack-Glare-Content-SHA1"])
        self.assertEqual('bbfd48c7ec792fc462e58232d4d9f407'
                         'ecefb75cc9e9823336166556b499ea4d',
                         response.headers["X-Openstack-Glare-Content-SHA256"])

        blob_data = self.get(url=url + '/blob')
        self.assertEqual(data, blob_data)

        # download artifact via admin
        self.set_user('admin')
        blob_data = self.get(url=url + '/blob')
        self.assertEqual(data, blob_data)

        # try to download blob via different user
        self.set_user('user2')
        self.get(url=url + '/blob', status=404)



# Copyright 2011-2016 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg
from oslo_context import context
from oslo_log import log as logging
from oslo_middleware import base as base_middleware
from oslo_middleware import request_id
from oslo_serialization import jsonutils

from glare.common import exception
from glare.common import policy
from glare.i18n import _

context_opts = [
    cfg.BoolOpt('allow_anonymous_access', default=False,
                help=_('Allow unauthenticated users to access the API with '
                       'read-only privileges. This only applies when using '
                       'ContextMiddleware.'))
]

CONF = cfg.CONF
CONF.register_opts(context_opts)

LOG = logging.getLogger(__name__)


class RequestContext(context.RequestContext):
    """Stores information about the security context for Glare.

    Stores how the user accesses the system, as well as additional request
    information.
    """

    def __init__(self, service_catalog=None, **kwargs):
        super(RequestContext, self).__init__(**kwargs)
        self.service_catalog = service_catalog
        # check if user is admin using policy file
        if kwargs.get('is_admin') is None:
            self.is_admin = policy.check_is_admin(self)

    def to_dict(self):
        d = super(RequestContext, self).to_dict()
        d.update({
            'service_catalog': self.service_catalog,
        })
        return d

    def to_policy_values(self):
        values = super(RequestContext, self).to_policy_values()
        values['is_admin'] = self.is_admin
        values['read_only'] = self.read_only
        return values


class BaseContextMiddleware(base_middleware.ConfigurableMiddleware):
    @staticmethod
    def process_response(resp, request=None):
        try:
            request_id = resp.request.context.request_id
            # For python 3 compatibility need to use bytes type
            prefix = b'req-' if isinstance(request_id, bytes) else 'req-'

            if not request_id.startswith(prefix):
                request_id = prefix + request_id

            resp.headers['x-openstack-request-id'] = request_id
        except AttributeError:
            pass

        return resp


class ContextMiddleware(BaseContextMiddleware):
    @staticmethod
    def process_request(req):
        """Convert authentication information into a request context.

        Generate a RequestContext object from the available
        authentication headers and store on the 'context' attribute
        of the req object.

        :param req: wsgi request object that will be given the context object
        :raises: webob.exc.HTTPUnauthorized: when value of the
                                            X-Identity-Status  header is not
                                            'Confirmed' and anonymous access
                                            is disallowed
        """
        if req.headers.get('X-Identity-Status') == 'Confirmed':
            req.context = ContextMiddleware._get_authenticated_context(req)
        elif CONF.allow_anonymous_access:
            req.context = RequestContext(read_only=True, is_admin=False)
        else:
            raise exception.Unauthorized()

    @staticmethod
    def _get_authenticated_context(req):
        headers = req.headers
        service_catalog = None
        if headers.get('X-Service-Catalog') is not None:
            catalog_header = headers.get('X-Service-Catalog')
            try:
                service_catalog = jsonutils.loads(catalog_header)
            except ValueError:
                raise exception.GlareException(
                    _('Invalid service catalog json.'))
        kwargs = {
            'service_catalog': service_catalog,
            'request_id': req.environ.get(request_id.ENV_REQUEST_ID),
        }
        return RequestContext.from_environ(req.environ, **kwargs)


class TrustedAuthMiddleware(BaseContextMiddleware):
    @staticmethod
    def process_request(req):
        auth_token = req.headers.get('X-Auth-Token')
        if not auth_token:
            msg = _("Auth token must be provided")
            raise exception.Unauthorized(msg)
        try:
            user, tenant, roles = auth_token.strip().split(':', 3)
        except ValueError:
            msg = _("Wrong auth token format. It must be 'user:tenant:roles'")
            raise exception.Unauthorized(msg)
        if not tenant:
            msg = _("Tenant must be specified in auth token. "
                    "Format of the token is 'user:tenant:roles'")
            raise exception.Unauthorized(msg)
        elif tenant.lower() == 'none':
            tenant = None
            req.headers['X-Identity-Status'] = 'Nope'
        else:
            req.headers['X-Identity-Status'] = 'Confirmed'

        req.headers['X-User-Id'] = user
        req.headers['X-Tenant-Id'] = tenant
        req.headers['X-Roles'] = roles

        if req.headers.get('X-Identity-Status') == 'Confirmed':
            kwargs = {'request_id': req.environ.get(request_id.ENV_REQUEST_ID)}
            req.context = RequestContext.from_environ(req.environ, **kwargs)
        elif CONF.allow_anonymous_access:
            req.context = RequestContext(read_only=True, is_admin=False)
        else:
            raise exception.Unauthorized()



# Copyright 2017 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from glare import engine
from glare.objects.meta import registry
from glare.tests.unit import base


class TestMultistore(base.BaseTestCase):

    def test_multistore(self):
        types = {'images': 'swift',
                 'heat_templates': 'rbd', 'heat_environments': 'file',
                 'tosca_templates': 'sheepdog',
                 'murano_packages': 'vsphere',
                 'sample_artifact': 'database',
                 'hooks_artifact': 'database',
                 'unpacking_artifact': 'database'}

        # create engine and register new artifact types
        engine.Engine()

        for type_name, store in types.items():
            self.config(default_store=store,
                        group='artifact_type:' + type_name)

        for t in registry.ArtifactRegistry.obj_classes().values():
            name = t[0].get_type_name()
            if name == 'all':
                continue
            self.assertEqual(
                getattr(base.CONF, 'artifact_type:' + name).default_store,
                types[name])
